using System;
using System.Collections.Generic;
using System.Text;
using System.IO;
using System.Threading;
using System.Security;
using System.Security.Principal;
using System.Diagnostics;
using System.Globalization;
using System.Runtime.Remoting;
using System.Runtime.Remoting.Channels;
using System.Runtime.Remoting.Channels.Tcp;
using System.Runtime.Remoting.Channels.Ipc;
using System.Runtime.Remoting.Messaging;
using System.Runtime.Serialization.Formatters;
using System.Runtime.Serialization.Formatters.Binary;
using System.Runtime.Serialization;
using System.Web.UI;

namespace Rainbird.AppServer.API
{
    /// <summary>
    /// Werkzeuge fr die Arbeit mit dem Applikationsserver.
    /// </summary>
    public static class ApplicationServer
    {
        #region Deklarationen

        /// <summary>
        /// Konstante fr eingebaute Rolle "Benutzer".
        /// </summary>
        public const string ROLE_USER = "User";

        /// <summary>
        /// Konstante fr eingebaute Rolle "Administrator".
        /// </summary>
        public const string ROLE_ADMINISTRATOR="Administrator";

        /// <summary>
        /// Ereignis: Erfolgreich angemeldet. 
        /// </summary>
        public static event EventHandler<SessionEventArgs> LogonSuccess;

        /// <summary>
        /// Ereignis: Anmeldung fehlgeschlagen.
        /// </summary>
        public static event EventHandler<SessionEventArgs> LogonFail;

        #endregion

        #region Sicherheit

        /// <summary>
        /// Gibt den Meldungstext "Zugriff verweigert." in der Sprache der Kultureinstellung des aktuellen Threads zurck.
        /// </summary>
        public static string LocalizedAccessDeniedMessage
        {
            get { return LanguageResource.AccessDenied; }
        }

        /// <summary>
        /// Meldet den aktuellen Benutzer am Applikationsserver an.
        /// </summary>
        /// <param name="serverURL">URL des Applikationsservers (z.B.: "tcp://localhost:6060/NTierExample/")
        /// <remarks>Achtung! Den Slash am Ende nicht vergessen!</remarks>
        /// </param>        
        /// <returns>Wahr, wenn die Anmeldung erfolgreich war, ansonsten Falsch</returns>
        public static bool Logon(string serverURL)
        {
            // Wenn kein Server-URL angegeben wurde ...
            if (string.IsNullOrEmpty(serverURL))
                // Ausnahme werfen
                throw new ArgumentNullException("serverURL");

            // Server-URL zerlegen
            string[] urlParts = serverURL.Split(':');

            // Protokoll lesen (Erster Teil des Server-URLs)
            string protocol = urlParts[0].ToLower();

            // Wenn den URL nicht mit / endet ...
            if (!serverURL.EndsWith(@"/"))
                // Ausnahme werfen
                throw new ArgumentException(string.Format(LanguageResource.InvalidServerURL, serverURL), "serverURL");

            // Kultureinstellungen des aktuellen Threads abrufen
            CultureInfo culture = CultureInfo.CurrentCulture;
            CultureInfo cultureUI = CultureInfo.CurrentUICulture;

            // URL zum Sicherheitsdienst zusammenbauen
            StringBuilder securityServiceURL = new StringBuilder();
            securityServiceURL.Append(serverURL);
            securityServiceURL.Append("Rainbird.AppServer.API.ISecurityService");

            // Variable fr den Sitzungsschlssel
            Guid sessionID = Guid.Empty;

            // Ggf. vorhandene Sitzungsinformationen im Aufrufkontext entfernen
            CallContext.FreeNamedDataSlot("session");

            try
            {
                // Protokoll auswerten
                switch (protocol)
                {
                    case "tcp": // TCP-Protokoll ber Netzwerk
                        // TCP-Client-Kanal konfigurieren
                        ConfigureClientTCPRemotingChannel(true, false);
                        
                        break;
                    case "ipc": // Named Pipes-Protokoll fr schnelle lokale Interprozess-Kommunikation
                        // IPC-Client-Kanal konfigurieren
                        ConfigureClientIPCRemotingChannel(true, false);

                        break;
                }
                // Verbindung zum Sicherheitsdienst des angegebenen Applikationsservers herstellen
                ISecurityService securityServiceProxy = (ISecurityService)Activator.GetObject(typeof(ISecurityService), securityServiceURL.ToString());

                // Benutzer anmelden und den Sitzungsschlssel lesen
                sessionID = securityServiceProxy.Logon();
            }
            catch (Exception ex)
            {
                // Wenn fr LogonFail Ereignisprozeduren registriert sind ...
                if (LogonFail != null)
                    // Ereignis feuern
                    LogonFail(null, new SessionEventArgs(ex.Message));

                // Falsch zurckgeben
                return false;
            }
            // Sitzungsinformationen in Containerobjekt verpacken, welches an den Aufrufkontext gehngt wird
            LogicalCallContextData contextData = new LogicalCallContextData(serverURL, sessionID,culture,cultureUI);
            CallContext.LogicalSetData("session", ContextDataToByteArray(contextData));

            // Wenn fr LogonSuccess Ereignisprozeduren registriert sind ...
            if (LogonSuccess != null)
                // Ereignis feuern
                LogonSuccess(null, new SessionEventArgs(string.Format(LanguageResource.LoggedOnSuccessfully, sessionID.ToString())));

            // Wahr zurckgeben
            return true;
        }

        /// <summary>
        /// Beendet die aktuelle Sitzung auf dem Applikationsserver.
        /// </summary>
        public static void Logoff()
        { 
            // Wenn momentan eine Sitzung besteht ...
            if (HasSession)
            {
                // Verbindung zum Sicherheitsdienst des Anmeldungs-Applikationsservers herstellen
                ISecurityService securityServiceProxy = (ISecurityService)Activator.GetObject(typeof(ISecurityService), ServerURL);

                // Sitzung abmelden
                securityServiceProxy.Logoff();

                // Sitzungsinformationen im Aufrufkontext entfernen
                CallContext.FreeNamedDataSlot("session");
            }
        }

        /// <summary>
        /// Fhrt eine Berechtigungsprfung durch.
        /// Geprft wird auf eine Rolle des rollenbasierten Sicherheitssystems.
        /// </summary>
        /// <param name="roleName">Rollenname</param>
        /// <returns>Ergebnis der Berechtigungsprfung</returns>
        public static bool IsInRole(string roleName)
        { 
            // Wenn momentan eine Sitzung besteht ...
            if (HasSession)
            {
                // Verbindung zum Sicherheitsdienst des Anmeldungs-Applikationsservers herstellen
                ISecurityService securityServiceProxy = (ISecurityService)Activator.GetObject(typeof(ISecurityService), ServerURL + typeof(ISecurityService).FullName);

                // Berechtigungsprfung durchfhren
                return securityServiceProxy.IsInRole(roleName);
            }
            // Falsch zurckgeben
            return false;
        }

        /// <summary>
        /// Prft, ob der Aufrufer vertrauenswrdig ist, oder nicht.
        /// </summary>
        /// <returns>Prfergebnis</returns>
        public static bool IsCallerTrustworthy
        {
            get
            {
                // Wenn momentan eine Sitzung besteht ...
                if (HasSession)
                {
                    // Verbindung zum Sicherheitsdienst des Anmeldungs-Applikationsservers herstellen
                    ISecurityService securityServiceProxy = (ISecurityService)Activator.GetObject(typeof(ISecurityService), ServerURL + typeof(ISecurityService).FullName);

                    // Wahr zurckgeben, wenn der Aufrufer vertrauenswrdig ist
                    return securityServiceProxy.IsCallerTrustworthy;
                }
                // Falsch zurckgeben
                return false;
            }
        }

        /// <summary>
        /// Gibt den Schlssel der aktuellen Sitzung zurck.
        /// </summary>
        public static Guid SessionID
        {
            get
            {
                // Wenn eine Sitzung vorhanden ist ...
                if (HasSession)
                {
                    // Angehngte Daten aus dem Aufrufkontext abrufen
                    LogicalCallContextData contextData = ByteArrayToContextData((byte[])CallContext.LogicalGetData("session"));

                    // Sitzungsschlssel zurckgeben
                    return contextData.SessionID;
                }
                else
                    // Leeren Sitzungsschlssel zurckgeben
                    return Guid.Empty;
            }
        }

        /// <summary>
        /// Gibt die Identitt des Benutzers zurck in dessen Kontext der aktuelle Thread luft.
        /// </summary>
        public static WindowsIdentity CurrentUser
        {
            get
            {
                // Identitt ermitteln und zurckgeben
                return (WindowsIdentity)Thread.CurrentPrincipal.Identity;
            }
        }

        /// <summary>
        /// Gibt den Sicherheits-Prinzipal des Benutzers zurck in dessen Kontext der aktuelle Thread luft.
        /// </summary>
        public static WindowsPrincipal CurrentSecurityPrincipal
        {
            get
            {
                // Prinzipal erzeugen
                WindowsPrincipal principal = new WindowsPrincipal(CurrentUser);

                // Prinzipal ermitteln und zurckgeben
                return principal;
            }
        }

        /// <summary>
        /// Gibt zurck, ob der Benutzer momentan an einem Applikationsserver angemeldet ist und dort
        /// eine Sitzung hat, oder nicht.
        /// </summary>
        public static bool HasSession
        {
            get
            {
                // Sitzungsinformationen aus dem Aufrufkontext lesen
                object contextData = CallContext.LogicalGetData("session");

                // Wahr zurckgeben, wenn es Sitzungsinformationen aus dem Aufrufkontext gelesen werden konnten ...
                return (contextData != null) && (contextData is byte[]);
            }
        }

        /// <summary>
        /// Gibt den URL des Applikationsservers zurck, an dem der Benutzer gerade angemeldet ist.
        /// </summary>
        public static string ServerURL
        {
            get
            {
                // Wenn eine Sitzung vorhanden ist ...
                if (HasSession)
                {
                    // Angehngte Daten aus dem Aufrufkontext abrufen
                    LogicalCallContextData contextData = ByteArrayToContextData((byte[])CallContext.LogicalGetData("session"));

                    // Server-URL zurckgeben
                    return contextData.ServerURL;
                }
                else
                    // Leeren String zurckgeben
                    return string.Empty;
            }
        }

        /// <summary>
        /// Serialisiert ein Kontextdatenobjekt in ein Byte-Array.
        /// </summary>
        /// <param name="contextData">Kontextdatenobjekt</param>
        /// <returns>Byte-Array</returns>
        private static byte[] ContextDataToByteArray(LogicalCallContextData contextData)
        {
            Debug.Assert(contextData!=null,"contextData!=null");

            // Binrformatierer erzeugen
            BinaryFormatter serializer = new BinaryFormatter();
            
            // Datenstrom im Hauptspeicher erzeugen
            MemoryStream stream=new MemoryStream();

            // Kontextdatenobjekt binr serialisieren
            serializer.Serialize(stream, contextData);
            stream.Flush();
                        
            // Serialisierungsergebnis (Byte-Array) lesen
            byte[] raw=stream.ToArray();

            // Datenstrom schlieen
            stream.Close();
            
            // Byte-Array zurckgeben
            return raw;
        }

        /// <summary>
        /// Deserialisiert ein Kontextdatenobjekt aus einem Byte-Array.
        /// </summary>
        /// <param name="raw">Byte-Array</param>
        /// <returns>Kontextdatenobjekt</returns>
        private static LogicalCallContextData ByteArrayToContextData(byte[] raw)
        {
            Debug.Assert(raw != null, "raw!=null");

            // Binrformatierer erzeugen
            BinaryFormatter serializer = new BinaryFormatter();

            // Datenstrom im Hauptspeicher erzeugen
            MemoryStream stream = new MemoryStream();

            // Byte-Array in den Datenstrom schieben
            stream.Write(raw, 0, raw.Length);
            stream.Flush();
            stream.Position = 0;

            // Kontextdatenobjekt binr serialisieren
            LogicalCallContextData contextData=(LogicalCallContextData)serializer.Deserialize(stream);
                        
            // Datenstrom schlieen
            stream.Close();

            // Kontextdatenobjekt zurckgeben
            return contextData;
        }

        /// <summary>
        /// Legt die Daten aus dem Aufrufkontext in eine Web-Sitzungsvariable.
        /// </summary>
        /// <param name="webPage">ASP.NET Webseite</param>
        public static void SaveContextDataInWebSessionState(Page webPage)
        { 
            Debug.Assert(webPage!=null,"webPage!=null");

            // Sitzungsinformationen aus dem Aufrufkontext holen
            object contextData=CallContext.LogicalGetData("session");

            // Wenn Sitzungsinformationen im Aufrufkontext gefunden wurden ...
            if (contextData!=null)
            {
                // Kontextdatenobjekt in Byte-Array umwandeln
                byte[] rawContextData=(byte[])contextData;

                // Kontextdaten-Byte-Array im Sitzungsstatus der Webseite speichern
                webPage.Session["RainbirdAppServerContextData"] = rawContextData;
            }
        }

        /// <summary>
        /// Ldt Sitzungsinformationen einer Web-Sitzungsvariable in den Aufrufkontext.
        /// </summary>
        /// <param name="webPage">ASP.NET Webseite</param>
        public static void LoadContextDataFromWebSessionState(Page webPage)
        {
            Debug.Assert(webPage != null, "webPage!=null");

            // Sitzungsinformationen aus dem Sitzungsstatus der Webseite holen
            object contextData = webPage.Session["RainbirdAppServerContextData"];

            // Wenn Sitzungsinformationen im Aufrufkontext gefunden wurden ...
            if (contextData != null)
            {
                // Kontextdatenobjekt in Byte-Array umwandeln
                byte[] rawContextData = (byte[])contextData;

                // Kontextdaten-Byte-Array in den Aufrufkontext schreiben
                CallContext.FreeNamedDataSlot("session");
                CallContext.LogicalSetData("session", rawContextData);
            }
        }

        #endregion

        #region Sperrenverwaltung

        /// <summary>
        /// Fordert eine Sperre fr eine bestimmte Ressource an.
        /// </summary>
        /// <param name="resourceName">Name der zu sperrenden Ressource</param>
        /// <param name="key">Schlssel der zu sperrenden Ressource</param>
        /// <returns>Informationen ber die Sperre</returns>
        public static LockingInfo AcquireLock(string resourceName, string key)
        {
            // Verbindung zum Sperrendienst herstellen
            ILockingService lockingServiceProxy=ServiceFactory<ILockingService>.CreateProxy();

            // Sperre anfordern und Sperren-Informationsobjekt zurckgeben
            return lockingServiceProxy.AcquireLock(resourceName, key);
        }

        /// <summary>
        /// Gibt eine Sperre fr eine bestimmte Ressource wieder frei.
        /// </summary>
        /// <param name="resourceName">Name der zu sperrenden Ressource</param>
        /// <param name="key">Schlssel der zu sperrenden Ressource</param>
        public static void ReleaseLock(string resourceName, string key)
        {
            // Verbindung zum Sperrendienst herstellen
            ILockingService lockingServiceProxy = ServiceFactory<ILockingService>.CreateProxy();

            // Sperre freigeben
            lockingServiceProxy.ReleaseLock(resourceName, key);
        }

        /// <summary>
        /// Gibt eine Sperre mit einem bestimmten Schlssel wieder frei.
        /// </summary>        
        /// <param name="lockID">Sperrschlssel</param>
        public static void ReleaseLock(Guid lockID)
        {
            // Verbindung zum Sperrendienst herstellen
            ILockingService lockingServiceProxy = ServiceFactory<ILockingService>.CreateProxy();

            // Sperre freigeben
            lockingServiceProxy.ReleaseLock(lockID);
        }

        /// <summary>
        /// Gibt Informationen ber eine bestehende Sperre einer bestimmten Ressource zurck.
        /// </summary>
        /// <param name="resourceName">Name der zu sperrenden Ressource</param>
        /// <param name="key">Schlssel der zu sperrenden Ressource</param>
        /// <returns>Informationen ber die Sperre</returns>
        public static LockingInfo GetLockInfo(string resourceName, string key)
        {
            // Verbindung zum Sperrendienst herstellen
            ILockingService lockingServiceProxy = ServiceFactory<ILockingService>.CreateProxy();

            // Sperr-Informationen der angegebenen Ressource abrufen und zurckgeben
            return lockingServiceProxy.GetLockInfo(resourceName, key);
        }

        /// <summary>
        /// Gibt eine Auflistung aller aktuellen Sperren zurck.
        /// </summary>
        /// <returns>Liste mit Sperren</returns>
        public static IDictionary<Guid, LockingInfo> GetLockList()
        {
            // Verbindung zum Sperrendienst herstellen
            ILockingService lockingServiceProxy = ServiceFactory<ILockingService>.CreateProxy();

            // Liste der aktuellen Sperren abrufen und zurckgeben
            return lockingServiceProxy.GetLockList();
        }

        #endregion

        #region Konfiguration der Client-Kommunikationskanle

        /// <summary>
        /// Richtet einen TCP-Clientkanal ein.
        /// </summary>      
        /// <param name="enableSecurity">Schalter fr Sicherheit</param>
        /// <param name="impersonate">Schalter fr Impersonierung</param>
        private static void ConfigureClientTCPRemotingChannel(bool enableSecurity, bool impersonate)
        {
            // Kanalnamen erzeugen
            string channelName = "RainbirdTCPClient";

            // Kanal suchen
            IChannel channel = ChannelServices.GetChannel(channelName);

            // Wenn der Kanal nicht gefunden wurde ...
            if (channel == null)
            {
                // Konfiguration fr den TCP-Kanal erstellen
                System.Collections.IDictionary channelSettings = new System.Collections.Hashtable();
                channelSettings["port"] = 0;
                channelSettings["name"] = channelName;
                channelSettings["secure"] = enableSecurity;

                // Wenn Sicherheit aktiviert ist ...
                if (enableSecurity)
                {
                    // Impersonierung entsprechend der Einstellung aktivieren oder deaktivieren
                    channelSettings["tokenImpersonationLevel"] = impersonate ? System.Security.Principal.TokenImpersonationLevel.Impersonation : System.Security.Principal.TokenImpersonationLevel.Identification;

                    // Signatur und Verschlssung explizit aktivieren
                    channelSettings["protectionLevel"] = System.Net.Security.ProtectionLevel.EncryptAndSign;
                }
                // Binre Serialisierung von komplexen Objekten aktivieren
                BinaryServerFormatterSinkProvider provider = new BinaryServerFormatterSinkProvider();
                provider.TypeFilterLevel = TypeFilterLevel.Full;

                // Neuen TCP-Kanal erzeugen
                channel = new TcpChannel(channelSettings, null, provider);

                // Kanal registrieren
                ChannelServices.RegisterChannel(channel, enableSecurity);
            }
        }

        /// <summary>
        /// Richtet einen IPC-Clientkanal ein.
        /// </summary>      
        /// <param name="enableSecurity">Schalter fr Sicherheit</param>
        /// <param name="impersonate">Schalter fr Impersonierung</param>
        private static void ConfigureClientIPCRemotingChannel(bool enableSecurity, bool impersonate)
        {
            // Kanalnamen erzeugen
            string channelName = "RainbirdIPCClient";

            // Kanal suchen
            IChannel channel = ChannelServices.GetChannel(channelName);

            // Wenn der Kanal nicht gefunden wurde ...
            if (channel == null)
            {
                // Konfiguration fr den IPC-Kanal erstellen
                System.Collections.IDictionary channelSettings = new System.Collections.Hashtable();
                channelSettings["name"] = channelName;
                //channelSettings["portName"] = "RainbirdIPC";
                channelSettings["secure"] = enableSecurity;

                // Wenn Sicherheit aktiviert ist ...
                if (enableSecurity)
                {
                    // Impersonierung entsprechend der Einstellung aktivieren oder deaktivieren
                    channelSettings["tokenImpersonationLevel"] = impersonate ? System.Security.Principal.TokenImpersonationLevel.Impersonation : System.Security.Principal.TokenImpersonationLevel.Identification;

                    // Signatur und Verschlssung explizit aktivieren
                    channelSettings["protectionLevel"] = System.Net.Security.ProtectionLevel.EncryptAndSign;
                }
                // Binre Serialisierung von komplexen Objekten aktivieren
                BinaryServerFormatterSinkProvider provider = new BinaryServerFormatterSinkProvider();
                provider.TypeFilterLevel = TypeFilterLevel.Full;

                // Neuen IPC-Kanal erzeugen
                channel = new IpcChannel(channelSettings, null, provider);

                // Kanal registrieren
                ChannelServices.RegisterChannel(channel, enableSecurity);
            }
        }

        #endregion

        #region Lokalisierung

        /// <summary>
        /// Legt die Kultureinstellungen der Applikationsserver.Sitzung fr den aktuellen Thread fest.
        /// </summary>
        public static void AssumeCultureFromSession()
        { 
            // Wenn eine Sitzung existiert ...
            if (HasSession)
            {
                // Angehngte Daten aus dem Aufrufkontext abrufen
                LogicalCallContextData contextData = ByteArrayToContextData((byte[])CallContext.LogicalGetData("session"));

                // Kultureinstellungen der Sitzung bernehmen
                Thread.CurrentThread.CurrentCulture = contextData.Culture;
                Thread.CurrentThread.CurrentUICulture = contextData.CultureUI;
            }
        }

        #endregion
    }
}
