using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.Common;

namespace Rainbird.AppServer.API
{
    /// <summary>
    /// Implementiert einfachen Datenzugriff auf Relationale Datenbanken.
    /// </summary>
    public class DataAccess
    {
        /// <summary>
        /// Speichert nderungen an einer Tabelle in der Datenbank.
        /// </summary>
        /// <param name="factory">Fabrik des Datenanbieters</param>
        /// <param name="connectionString">Verbindungszeichenfolge</param>
        /// <param name="table">Tabelle mit nderungen</param>
        public static void Update(DbProviderFactory factory, string connectionString, DataTable table)
        {
            // Datenbankverbindung erzeugen
            using (DbConnection connection = factory.CreateConnection())
            {
                // Verbindungszeichenfolge bernehmen
                connection.ConnectionString = connectionString;

                // SELECT-Befehl zusammensetzen
                StringBuilder statement = new StringBuilder();
                statement.Append("SELECT * FROM ");
                statement.Append(table.TableName);
                statement.Append(" WHERE 0=1");

                // Datenbankbefehl erzeugen
                DbCommand selectCommand = factory.CreateCommand();
                selectCommand.Connection = connection;
                selectCommand.CommandType = CommandType.Text;
                selectCommand.CommandText = statement.ToString();

                // Datenadapter erzuegen
                DbDataAdapter adapter = factory.CreateDataAdapter();
                adapter.SelectCommand = selectCommand;
                adapter.AcceptChangesDuringUpdate = true;

                // Befehlsgenerator erzeugen
                DbCommandBuilder builder = factory.CreateCommandBuilder();
                builder.ConflictOption = ConflictOption.CompareAllSearchableValues;
                builder.DataAdapter = adapter;

                // nderungen bernehmen
                adapter.Update(table);
            }
        }

        /// <summary>
        /// Fhrt eine Datenbankabfrage aus.
        /// </summary>
        /// <param name="factory">Fabrik des Datenbanbieters</param>
        /// <param name="connectionString">Verbindungszeichenfolge zur Datenbank</param>
        /// <param name="query">SQL-SELECT-Abfrage</param>
        /// <param name="parameters">Paramaterliste (Optional)</param>
        /// <returns>Tabelle mit Abfrageergebnissen</returns>
        public static DataTable Query(DbProviderFactory factory, string connectionString, string query, IList<DbParameter> parameters)
        {
            // Tabellenname ermitteln
            string tableName = GetTableNameFromQuery(query);

            // Tabelle erzeugen
            DataTable table = new DataTable(tableName);

            // Aufruf delegieren
            return Query(factory, table, connectionString, query, parameters);
        }

        /// <summary>
        /// Fhrt eine Datenbankabfrage aus.
        /// </summary>
        /// <param name="factory">Fabrik des Datenbanbieters</param>
        /// <param name="table">Tabelle</param>
        /// <param name="connectionString">Verbindungszeichenfolge zur Datenbank</param>
        /// <param name="query">SQL-SELECT-Abfrage</param>
        /// <param name="parameters">Paramaterliste (Optional)</param>
        /// <returns>Tabelle mit Abfrageergebnissen</returns>
        public static DataTable Query(DbProviderFactory factory, DataTable table, string connectionString, string query, IList<DbParameter> parameters)
        {
            // Wenn keine Tabelle bergeben wurde ...
            if (table == null)
                // Neue Tabelle erzeugen
                table = new DataTable();

            // Datenbankverbindung erzeugen
            using (DbConnection connection = factory.CreateConnection())
            {
                // Verbindungszeichenfolge bernehmen
                connection.ConnectionString = connectionString;

                // Datenbankbefehl erzeugen
                DbCommand command = factory.CreateCommand();
                command.Connection = connection;
                command.CommandType = CommandType.Text;
                command.CommandText = query;

                // Wenn Parameter angegeben wurden ...
                if (parameters != null)
                {
                    // Alle Parameter durchlaufen
                    foreach (DbParameter parameter in parameters)
                    {
                        // Parameter zufgen
                        command.Parameters.Add(parameter);
                    }
                }
                // Datenadapter erzeugen
                DbDataAdapter adapter = factory.CreateDataAdapter();
                adapter.SelectCommand = command;

                // Abfrage ausfhren
                adapter.Fill(table);
            }
            // Tabelle zurckgeben
            return table;
        }

        /// <summary>
        /// Ermittelt den Tabellennamen fr eine DataTable aus einer SQL-Abfrage.
        /// </summary>
        /// <param name="query">SQL-Abfrage</param>
        /// <returns>Tabellenname</returns>
        public static string GetTableNameFromQuery(string query)
        {
            // SQL-Abfrage zerlegen
            string[] parts = query.Split(' ');

            // Alle Teile durchlaufen
            for (int i = 0; i < parts.Length; i++)
            {
                // Wenn der aktuelle Teil "FROM" ist ...
                if (parts[i].ToLower() == "from")
                {
                    // Tabellenname lesen und zurckgeben
                    return parts[i + 1].Split(',')[0];
                }
            }
            // "Table" zurckgeben
            return "Table";
        }

        /// <summary>
        /// Erzeugt einen Parameter fr eine Datenbankabfrage.
        /// </summary>
        /// <param name="factory">Fabrik des Datenbanbieters</param>
        /// <param name="name">Name des Parameters</param>
        /// <param name="type">Datentyp</param>
        /// <param name="size">Gre</param>
        /// <param name="direction">Richtung (Eingabe- oder Ausgabeparameter)</param>
        /// <param name="value">Wert</param>
        /// <returns>Fertig konfigurierter Parameter</returns>
        public static DbParameter CreateParameter(DbProviderFactory factory, string name, DbType type, int size, ParameterDirection direction, object value)
        {
            // Neuen Parameter erzeugen
            DbParameter parameter = factory.CreateParameter();
            parameter.ParameterName = name;
            parameter.DbType = type;
            parameter.Size = size;
            parameter.Direction = direction;
            parameter.Value = value;

            // Parameter zurckgeben
            return parameter;
        }

        /// <summary>
        /// Erzeugt einen Parameter fr eine Datenbankabfrage.
        /// </summary>
        /// <param name="factory">Fabrik des Datenbanbieters</param>
        /// <param name="name">Name des Parameters</param>
        /// <param name="type">Datentyp</param>        
        /// <param name="direction">Richtung (Eingabe- oder Ausgabeparameter)</param>
        /// <param name="value">Wert</param>
        /// <returns>Fertig konfigurierter Parameter</returns>
        public static DbParameter CreateParameter(DbProviderFactory factory, string name, DbType type, ParameterDirection direction, object value)
        {
            // An andere berladung delegieren
            return CreateParameter(factory, name, type, 0, direction, value);
        }

        /// <summary>
        /// Erzeugt einen Parameter fr eine Datenbankabfrage.
        /// </summary>
        /// <param name="factory">Fabrik des Datenbanbieters</param>
        /// <param name="name">Name des Parameters</param>
        /// <param name="type">Datentyp</param>        
        /// <param name="direction">Richtung (Eingabe- oder Ausgabeparameter)</param>        
        /// <returns>Fertig konfigurierter Parameter</returns>
        public static DbParameter CreateParameter(DbProviderFactory factory, string name, DbType type, ParameterDirection direction)
        {
            // An andere berladung delegieren
            return CreateParameter(factory, name, type, 0, direction, DBNull.Value);
        }

        /// <summary>
        /// Erzeugt einen Parameter fr eine Datenbankabfrage.
        /// </summary>
        /// <param name="factory">Fabrik des Datenbanbieters</param>
        /// <param name="name">Name des Parameters</param>
        /// <param name="type">Datentyp</param>   
        /// <param name="size">Gre</param>
        /// <param name="direction">Richtung (Eingabe- oder Ausgabeparameter)</param>        
        /// <returns>Fertig konfigurierter Parameter</returns>
        public static DbParameter CreateParameter(DbProviderFactory factory, string name, DbType type, int size, ParameterDirection direction)
        {
            // An andere berladung delegieren
            return CreateParameter(factory, name, type, size, direction, DBNull.Value);
        }
    }
}
