using System;
using System.Collections.Generic;
using System.Runtime.Remoting;
using System.Text;

namespace Rainbird.AppServer.API
{
    /// <summary>
    /// Fabrik fr Dienst-Proxies.
    /// </summary>
    /// <typeparam name="T">Schnittstelle des Dienstes, fr den ein Proxy erzeug werden soll</typeparam>
    public static class ServiceFactory <T>
    {
        /// <summary>
        /// Erzeugt ein Stellvertreter-Objekt (Proxy) zur Kommunikation mit einem entfernten Geschftsdienst.
        /// <remarks>
        /// Diese berladung verwendet den Anmelde-Server-URL der Sitzung.
        /// </remarks>
        /// </summary>
        /// <returns>Proxy</returns>
        public static T CreateProxy()
        { 
            // Wenn noch keine Sitzung existiert ...
            if (!ApplicationServer.HasSession)
                // Ausnahme werfen
                throw new ApplicationException(LanguageResource.NoSession);

            // Aufruf an berladung delegieren
            return CreateProxy(ApplicationServer.ServerURL);
        }

        /// <summary>
        /// Erzeugt ein Stellvertreter-Objekt (Proxy) zur Kommunikation mit einem entfernten Geschftsdienst.
        /// </summary>
        /// <param name="serverURL">URL zum Applikationsserver</param>
        /// <returns>Proxy</returns>
        public static T CreateProxy(string serverURL)
        { 
            // Metadaten der Schnittstelle abrufen
            Type interfaceType = typeof(T);

            // URL zur Adressierung des entfernten Dienstes zusammenbauen
            StringBuilder urlBuilder = new StringBuilder();
            urlBuilder.Append(serverURL);
            urlBuilder.Append(interfaceType.FullName);

            try
            {
                // Proxy erzeugen und zurckgeben
                return (T)Activator.GetObject(interfaceType, urlBuilder.ToString());
            }
            catch (Exception ex)
            { 
                // Ausnahme einpacken und weiterwerfen
                throw new ArgumentException(string.Format(LanguageResource.ErrorCreatingProxy, interfaceType.FullName), ex);
            }            
        }

        /// <summary>
        /// Erzeugt ein Stellvertreter-Objekt (Proxy) zur Kommunikation mit einem Geschftsdienst auf dem lokalen 
        /// Applikationssserver ber den IPC-Kanal.
        /// <remarks>
        /// Diese Methode funktioniert nicht fr entfernte Clients!
        /// </remarks>
        /// </summary>
        /// <returns>Proxy</returns>
        public static T CreateLocalProxyViaIPC()
        {             
            // Metadaten der Schnittstelle abrufen
            Type interfaceType = typeof(T);

            // IPC-URL zusammenbauen
            StringBuilder urlBuilder = new StringBuilder();
            urlBuilder.Append(@"ipc://RainbirdIPC/");
            urlBuilder.Append(RemotingConfiguration.ApplicationName); // Name der Remoting-Anwendung verwenden
            urlBuilder.Append(@"/");
            urlBuilder.Append(interfaceType.FullName);

            try
            {
                // Proxy erzeugen und zurckgeben
                return (T)Activator.GetObject(interfaceType, urlBuilder.ToString());
            }
            catch (Exception ex)
            {
                // Ausnahme einpacken und weiterwerfen
                throw new ArgumentException(string.Format(LanguageResource.ErrorCreatingProxy, interfaceType.FullName), ex);
            }          
        }
    }
}
