using System;
using System.Collections.Generic;
using System.Text;
using System.Diagnostics;
using System.Timers;
using System.Security;
using System.Security.Principal;
using Rainbird.AppServer.API;
using System.Runtime.Remoting.Messaging;

namespace Rainbird.AppServer.Locking
{
    /// <summary>
    /// Implementierung des Sperrungsdienstes.
    /// </summary>
    public class LockingService : ContextBoundObject, ILockingService, IDisposable
    {
        #region Deklarationen

        // Ablaufverfolgungsobjekt
        private TraceSource _traceSource = null;

        // Sperrliste
        IDictionary<Guid, LockingInfo> _lockList = null;

        // Suchindex (Um Sperren anahnd von RessourcenName und Ressourcen-Schlssel schneller zu finden)
        IDictionary<string, Guid> _nameSearchIndex = null;

        // Maximale Lebensdauer einer Sperre (in Minuten)
        private int _maxLockLifetime = 0;

        // Zeitgeber
        Timer _timer = null;

        #endregion

        #region Singleton-Implementierung

        // Singleton-Dienstinstanz
        private static LockingService _singleton = null;

        /// <summary>
        /// Privater Standardkonstruktor.
        /// </summary>
        private LockingService() 
        {
            // Ablaufverfolgungsobjekt erzeugen
            _traceSource = new TraceSource("LockingService");

            // Sperrliste erzeugen
            _lockList = new Dictionary<Guid, LockingInfo>();

            // Suchindex-Wrterbuch erzeugen
            _nameSearchIndex=new Dictionary<string,Guid>();

            // Wenn die Zeitgesteuerte Sperren-Verfallsprfung in der Konfiguration eingeschaltet ist ...
            if (Properties.Settings.Default.ReleaseExpiredLocks)
            {
                // Intervall aus Konfiguration abrufen
                double interval = Properties.Settings.Default.LockingTimerInterval;

                Debug.Assert(interval > 100, "interval > 100");

                // Maximale Sperren-Lebensdazer aus der Konfiguration abrufen
                _maxLockLifetime = Properties.Settings.Default.MaxLockLifetime;

                Debug.Assert(_maxLockLifetime > 0, "_maxLockLifetime > 0");

                // Zeitgeber erzeugen, Ereignis abonnieren und starten
                _timer = new Timer(interval);
                _timer.Elapsed += new ElapsedEventHandler(_timer_Elapsed);
                _timer.Start();
            }
        }

        /// <summary>
        /// Gibt die Singleton-Instanz des Sperrungsdienstes zurck.
        /// </summary>
        public static LockingService Instance
        {
            get
            { 
                // Wenn die Instanz noch nicht existiert ...
                if (_singleton == null)
                    // Instanz erstellen
                    _singleton = new LockingService();

                // Singleton-Instanz zurckgeben
                return _singleton;
            }
        }

        #endregion

        #region Zeitgesteuerte Sperren-Verfallsprfung

        /// <summary>
        /// Wird nach jedem Zeitgeber-Intervall aufgerufen.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _timer_Elapsed(object sender, ElapsedEventArgs e)
        {
            // Sperrliste bereinigen
            SweepLockList(e.SignalTime);
        }

        /// <summary>
        /// Bereinigt die Sperrliste von abgelaufenen Sperren.
        /// </summary>
        /// <param name="signalTime">Genauer Zeitpunkt des Zeitgeber-Intervalls</param>
        private void SweepLockList(DateTime signalTime)
        {             
            // Sperrliste fr gleichzeitigen Zugriff aus anderen Threads sperren
            lock (_lockList)
            { 
                // Liste fr abgelaufene Sperren erzeugen
                IList<Guid> expiredLocks = new List<Guid>();

                // Alle Sperren durchlaufen
                foreach (LockingInfo currentLock in _lockList.Values)
                {
                    // Wenn die Sperre abgelaufen ist ...
                    if (currentLock.Timestamp.AddMinutes(_maxLockLifetime) < signalTime)
                        // Auf die Liste setzen
                        expiredLocks.Add(currentLock.LockID);
                }
                // Alle abgelaufenen Sperren durchlaufen
                foreach (Guid lockID in expiredLocks)
                {
                    // Sperre aus der Sperrliste entfernen
                    _lockList.Remove(lockID);
                }
            }
        }

        #endregion

        #region ILockingService Member

        /// <summary>
        /// Fordert eine Sperre fr eine bestimmte Ressource an.
        /// </summary>
        /// <param name="resourceName">Name der zu sperrenden Ressource</param>
        /// <param name="key">Schlssel der zu sperrenden Ressource</param>
        /// <returns>Informationen ber die Sperre</returns>
        public LockingInfo AcquireLock(string resourceName, string key)
        {
            Debug.Assert(!string.IsNullOrEmpty(resourceName), "!string.IsNullOrEmpty(resourceName)");
            Debug.Assert(!string.IsNullOrEmpty(key), "!string.IsNullOrEmpty(key)");

            _traceSource.TraceEvent(TraceEventType.Information, 3101, "AcquireLock: User={0} Resource='{1}' Key='{2}'",ApplicationServer.CurrentUser.Name, resourceName, key);

            // Wenn der Aufrufer vertrauenswrdig ist ...
            if (ApplicationServer.IsCallerTrustworthy)
            {
                // Kultureinstellungen der Sitzung bernehmen
                ApplicationServer.AssumeCultureFromSession();

                // Sperrliste fr gleichzeitigen Zugriff aus anderen Threads sperren
                lock (_lockList)
                { 
                    // Nach bestehender Sperre fr die angegebene Ressource suchen
                    LockingInfo existingLock = GetLock(resourceName, key);

                    // Wenn bereits eine Sperre besteht ...
                    if (existingLock != null)
                    {
                        _traceSource.TraceEvent(TraceEventType.Verbose, 3001, "-> Result: {0}", false.ToString());

                        // Werte der bestehenden Sperre in Rckgabenachricht kopieren, Success auf Falsch setzen und zurckgeben
                        return new LockingInfo(existingLock.LockID, existingLock.ResourceName, existingLock.Key, existingLock.UserName, existingLock.Timestamp, false, string.Format(LanguageResource.ResourceAlreadyLocked,existingLock.UserName));
                    }
                    // Windows-Identitt des aktuellen Threads abrufen
                    WindowsIdentity identity = ApplicationServer.CurrentUser;

                    // Neue Sperre erzeugen
                    LockingInfo lockEntry = new LockingInfo(Guid.NewGuid(), resourceName, key, identity.Name, DateTime.Now, true, "ok");

                    // Neue Sperre in Sperrliste eintragen
                    _lockList.Add(lockEntry.LockID, lockEntry);

                    // Namen in den Suchindex einragen
                    _nameSearchIndex.Add(BuildSearchKey(resourceName, key), lockEntry.LockID);

                    _traceSource.TraceEvent(TraceEventType.Verbose, 3001, "-> Result: {0}",lockEntry.Success.ToString());

                    // Sperrinformationen zurckgeben
                    return lockEntry;
                }
            }
            // Nichts zurckgeben
            return null;
        }

        /// <summary>
        /// Gibt eine Sperre fr eine bestimmte Ressource wieder frei.
        /// </summary>
        /// <param name="resourceName">Name der zu sperrenden Ressource</param>
        /// <param name="key">Schlssel der zu sperrenden Ressource</param>
        public void ReleaseLock(string resourceName, string key)
        {
            Debug.Assert(!string.IsNullOrEmpty(resourceName), "!string.IsNullOrEmpty(resourceName)");
            Debug.Assert(!string.IsNullOrEmpty(key), "!string.IsNullOrEmpty(key)");

            _traceSource.TraceEvent(TraceEventType.Information, 3102, "ReleaseLock: User={0} Resource='{1}' Key='{2}'", ApplicationServer.CurrentUser.Name, resourceName, key);

            // Suchschlssel erzeugen
            string searchKey = BuildSearchKey(resourceName, key);

            // Sperrschlssel
            Guid lockID = Guid.Empty;

            // Wenn die Kombination aus Ressourcenname und -schlssel im Suchindex eingetragen ist ...
            if (_nameSearchIndex.TryGetValue(searchKey, out lockID))
            {
                // Sperre aufheben
                ReleaseLock(lockID);
            }
        }

        /// <summary>
        /// Gibt eine Sperre mit einem bestimmten Schlssel wieder frei.
        /// </summary>        
        /// <param name="lockID">Sperrschlssel</param>
        public void ReleaseLock(Guid lockID)
        {
            Debug.Assert(lockID != null, "lockID != null");

            _traceSource.TraceEvent(TraceEventType.Information, 3102, "ReleaseLock: User={0} Lock ID='{1}'", ApplicationServer.CurrentUser.Name, lockID.ToString());

            // Wenn der Aufrufer vertrauenswrdig ist ...
            if (ApplicationServer.IsCallerTrustworthy)
            {
                // Kultureinstellungen der Sitzung bernehmen
                ApplicationServer.AssumeCultureFromSession();

                // Sperrliste fr gleichzeitigen Zugriff aus anderen Threads sperren
                lock (_lockList)
                {
                    // Sperre abrufen
                    LockingInfo lockInfo = _lockList[lockID];

                    // Wenn der Benutzer stimmt ...
                    if (ApplicationServer.CurrentUser.Name == lockInfo.UserName)
                    {
                        // Suchschlssel erzeugen
                        string searchKey = BuildSearchKey(lockInfo.ResourceName, lockInfo.Key);

                        // Sperre aus Suchindex entfernen
                        _nameSearchIndex.Remove(searchKey);

                        // Sperre aus Sperrliste entfernen
                        _lockList.Remove(lockID);
                    }
                    else
                        // Ausnahme werfen
                        throw new SecurityException();
                }
            }
            else
                // Ausnahme werfen
                throw new SecurityException(LanguageResource.NoSession);
        }

        /// <summary>
        /// Gibt Informationen ber eine bestehende Sperre einer bestimmten Ressource zurck.
        /// </summary>
        /// <param name="resourceName">Name der zu sperrenden Ressource</param>
        /// <param name="key">Schlssel der zu sperrenden Ressource</param>
        /// <returns>Informationen ber die Sperre</returns>
        public LockingInfo GetLockInfo(string resourceName, string key)
        {
            Debug.Assert(!string.IsNullOrEmpty(resourceName), "!string.IsNullOrEmpty(resourceName)");
            Debug.Assert(!string.IsNullOrEmpty(key), "!string.IsNullOrEmpty(key)");

            _traceSource.TraceEvent(TraceEventType.Information, 3103, "GetLockInfo: User={0} Resource='{1}' Key='{2}'", ApplicationServer.CurrentUser.Name, resourceName, key);

            // Wenn der Aufrufer vertrauenswrdig ist ...
            if (ApplicationServer.IsCallerTrustworthy)
            {
                // Kultureinstellungen der Sitzung bernehmen
                ApplicationServer.AssumeCultureFromSession();

                // Informationen ber die Sperre zurckgeben
                return GetLock(resourceName, key);
            }
            else
                // Ausnahme werfen
                throw new SecurityException(LanguageResource.NoSession);
        }

        /// <summary>
        /// Gibt eine Auflistung aller aktuellen Sperren zurck.
        /// </summary>
        /// <returns>Liste mit Sperren</returns>
        public IDictionary<Guid, LockingInfo> GetLockList()
        {
            _traceSource.TraceEvent(TraceEventType.Information, 3104, "GetLockList.");

            // Wenn der Aufrufer "Administrator" ist ...
            if (ApplicationServer.IsInRole(ApplicationServer.ROLE_ADMINISTRATOR))
            {
                // Kultureinstellungen der Sitzung bernehmen
                ApplicationServer.AssumeCultureFromSession();

                // Schlsselliste zurckgeben
                return _lockList;       
            }
            // Nichts zurckgeben
            return null;
        }

        #endregion

        #region Hilfsfunktionen

        /// <summary>
        /// Erzeugt aus Ressourcenname und Resourcenschlssel einen Suchschlssel fr die Verwendung mit dem Namen-Suchindex.
        /// </summary>
        /// <param name="resourceName">Name der zu sperrenden Ressource</param>
        /// <param name="key">Schlssel der zu sperrenden Ressource</param>
        /// <returns>Suchschlssel</returns>
        private string BuildSearchKey(string resourceName, string key)
        {
            // Such-Zeichenfolge erzeugen
            StringBuilder builder = new StringBuilder();
            builder.Append(resourceName);
            builder.Append('|');
            builder.Append(key);
            return builder.ToString();
        }

        /// <summary>
        /// Gibt Informationen ber eine bestehende Sperre einer bestimmten Ressource zurck.
        /// </summary>
        /// <param name="resourceName">Name der zu sperrenden Ressource</param>
        /// <param name="key">Schlssel der zu sperrenden Ressource</param>
        /// <returns>Informationen ber die Sperre</returns>
        public LockingInfo GetLock(string resourceName, string key)
        {
            // Sperrliste fr gleichzeitigen Zugriff aus anderen Threads sperren
            lock (_lockList)
            {
                // Suchschlssel erzeugen
                string searchKey = BuildSearchKey(resourceName, key);

                // Sperrschlssel
                Guid lockID = Guid.Empty;

                // Wenn die Kombination aus Ressourcenname und -schlssel im Suchindex eingetragen ist ...
                if (_nameSearchIndex.TryGetValue(searchKey, out lockID))
                {
                    // Sperren-Informationsobjekt zurckgeben
                    return _lockList[lockID];
                }
                else
                    // Keine Sperre auf diese Ressource gesetzt, deshalb nichts zurckgeben
                    return null;
            }
        }

        #endregion

        #region IDisposable Member

        /// <summary>
        /// Verwendete Ressourcen sofort freigeben.
        /// </summary>
        public void Dispose()
        {
            // Wenn der Zeitgeber noch existiert ...
            if (_timer != null)
            {
                // Zeitgeber entsorgen
                _timer.Dispose();
                _timer = null;
            }
            // Wenn die Sperrliste noch existiert ...
            if (_lockList != null)
            {
                // Sperrliste leeren
                _lockList.Clear();
                _lockList = null;
            }
            // Wenn der Namens-Suchindex noch existiert ...
            if (_nameSearchIndex != null)
            { 
                // Namens-Suchindex leeren
                _nameSearchIndex.Clear();
                _nameSearchIndex = null;
            }
        }

        #endregion

        #region Steuerung der Lebenszeit

        /// <summary>
        /// Ruft ein Lebensdauerdienstobjekt ab, mit dem die Lebensdauerrichtlinien fr
        ///  diese Instanz gesteuert werden knnen.
        /// </summary>
        /// <returns>Lebensdauerdienstobjekt</returns>
        public override object InitializeLifetimeService()
        {
            // Nichts zurckgeben, damit der Dienst ewig lebt.
            return null;
        }

        #endregion
    }
}
