using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.Remoting.Messaging;
using System.Globalization;
using System.Security;
using System.Security.Principal;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Reflection;
using System.Threading;
using System.Diagnostics;
using Rainbird.AppServer.API;

namespace Rainbird.AppServer
{
    /// <summary>
    /// Implementierung des Sicherheitsdienstes.
    /// </summary>
    public class SecurityService : ContextBoundObject, ISecurityService
    {
        #region Deklarationen

        // Ablaufverfolgungsobjekt
        private TraceSource _traceSource = null;

        // Rollenspeicher
        private RoleStore _roleStore = null;

        // Liste der eingebauten Rollen
        private IList<string> _builtInRoles = null;

        // Auflistung der aktuellen Sitzungen
        private IDictionary<Guid, Session> _sessions = null;

        // Index zur Suche von Sitzungen anhand des Benutzernamens
        private IDictionary<string, Guid> _userNameIndex = null;

        #endregion

        #region Singleton-Implementierung

        // Singleton-Dienstinstanz
        private static SecurityService _singleton = null;

        /// <summary>
        /// Privater Standardkonstruktor.
        /// </summary>
        private SecurityService()
        {
            // Ablaufverfolgungsobjekt erzeugen
            _traceSource = new TraceSource("SecurityService");

            _traceSource.TraceEvent(TraceEventType.Start, 2001, "Begin construction.");

            // Liste der eingebauten Rollen erzeugen
            _builtInRoles = new List<string>();
            _builtInRoles.Add(ApplicationServer.ROLE_USER); // Benutzer-Rolle (Ermglicht Anmeldung!)
            _builtInRoles.Add(ApplicationServer.ROLE_ADMINISTRATOR); // Administrator-Rolle (Ermglicht Verwaltungszugriff!)

            // Sitzungsliste erzeugen
            _sessions = new Dictionary<Guid, Session>();

            // Benutzernamen-Suchindexliste erzeugen
            _userNameIndex = new Dictionary<string, Guid>();

            // Wenn eine Rollenspeicherdatei existiert ...
            if (File.Exists(RoleStoreFileName))
                // Rollenspeicher laden
                LoadRoleStore();
            else
                // Neuen leeren Rollenspeicher erzeugen
                _roleStore = new RoleStore();

            _traceSource.TraceEvent(TraceEventType.Stop, 2002, "Construction complete.");
        }

        /// <summary>
        /// Gibt die Singleton-Instanz des Sicherheitsdienstes zurck.
        /// </summary>
        public static SecurityService Instance
        {
            get
            {
                // Wenn die Instanz noch nicht existiert ...
                if (_singleton == null)
                    // Instanz erstellen
                    _singleton = new SecurityService();

                // Singleton-Instanz zurckgeben
                return _singleton;
            }
        }

        #endregion

        #region ISecurityService Member

        /// <summary>
        /// Gibt zurck, ob die aktuelle Sitzung gltig ist, oder nicht.
        /// </summary>
        public bool IsSessionValid
        {
            get
            {
                // Wenn der Aufrufer korrekt authentifiziert ist ...
                if (ApplicationServer.CurrentUser.IsAuthenticated)
                {
                    // Sitzungsschlssel aus dem Aufrufkontext lesen
                    Guid sessionID = ApplicationServer.SessionID;

                    // Wenn dem Aufrufkontext keine Sitzungsinformationen angehngt sind ...
                    if (sessionID == Guid.Empty)
                        // Falsch zurckgeben
                        return false;

                    // Sitzungsobjekt
                    Session session = null;

                    // Wenn eine Sitzung mit dem Schlssel aus dem Aufrufkontext existiert ...
                    if (_sessions.TryGetValue(sessionID, out session))
                    {
                        // Wenn die Identitt des Aufrufers nicht mit der Indentitt der bestehenden Sitzung bereinstimmt ...
                        if (session.Identity.Name != ApplicationServer.CurrentUser.Name)
                            // Falsch zurckgeben
                            return false;

                        // Sitzung prfen und Ergbnis der Prfung zurckgeben
                        return !IsSessionExpired(session);
                    }
                }
                else
                    _traceSource.TraceEvent(TraceEventType.Error, 2302, "Caller is not authenticated.");

                // Falsch zurckgeben
                return false;
            }
        }

        /// <summary>
        /// Meldet einen Benutzer am Applikationsserver an und erffnet eine neue Sitzung.
        /// </summary>
        /// <returns>Sitzungsschlssel</returns>
        public Guid Logon()
        {
            _traceSource.TraceEvent(TraceEventType.Information, 2102, "Logon.");
            _traceSource.TraceEvent(TraceEventType.Verbose, 2011, "-> User: {0}",ApplicationServer.CurrentUser.Name);

            // Wenn der Aufrufer sich korrekt ausweisen kann ...
            if (PassportCheck())
            {
                // Ggf. bestehende Sitzung lesen
                Guid sessionID = ApplicationServer.SessionID;

                // Sitzungsliste fr den gleichzeitigen Zugriff mehrerer Threads sperren
                lock (_sessions)
                {
                    // Wenn der Benutzer bereits angemeldet ist ...
                    if (sessionID != Guid.Empty)
                    {
                        _traceSource.TraceEvent(TraceEventType.Verbose, 2008, "Existing session detected: {0}",sessionID.ToString());

                        // Schalter fr Ergebnis der Identittsprfung beim Versuch die Sitzung zu erneuern
                        bool wrongIdentity=false;

                        // Wenn die bestehende Sitzung nicht erneuert werden konnte ...
                        if (!RenewSession(sessionID, out wrongIdentity))
                        {
                            // Wenn die Identitt stimmt ...
                            if (!wrongIdentity)
                            {
                                // Benutzernamen Sucheintrag lschen
                                _userNameIndex.Remove(new KeyValuePair<string, Guid>(_sessions[sessionID].Identity.Name, sessionID));

                                // Sitzung lschen
                                _sessions.Remove(sessionID);
                            }                              

                            _traceSource.TraceEvent(TraceEventType.Error, 2305, "Logon failed.");

                            // Leeren Sitzungsschlssel zurckgeben
                            return Guid.Empty;
                        }
                        _traceSource.TraceEvent(TraceEventType.Verbose, 2006, "Logon successful.");

                        // Sitzungsschlssel zurckgeben
                        return sessionID;
                    }                
                    // Identitt des Aufrufers ermitteln
                    WindowsIdentity identity = ApplicationServer.CurrentUser;

                    // Wenn bereits eine andere Sitzung fr diesen Benutzer existiert ... 
                    if (_userNameIndex.TryGetValue(identity.Name, out sessionID))
                    {
                        // Wenn die Sitzung abgelaufen ist ...
                        if (IsSessionExpired(sessionID))
                        {
                            _traceSource.TraceEvent(TraceEventType.Information, 2101, "Session is expired.");

                            // Wenn die Sitzung nicht erneuert werden konnte ...
                            if (!RenewSession(sessionID))
                            {                                
                                // Benutzernamen Sucheintrag lschen
                                _userNameIndex.Remove(new KeyValuePair<string, Guid>(_sessions[sessionID].Identity.Name, sessionID));

                                // Sitzung lschen
                                _sessions.Remove(sessionID);

                                _traceSource.TraceEvent(TraceEventType.Error, 2305, "Logon failed.");

                                // Leeren Sitzungsschlssel zurckgeben
                                return Guid.Empty;
                            }
                        }
                        _traceSource.TraceEvent(TraceEventType.Verbose, 2006, "Logon successful.");

                        // Sitzungsschlssel zurckgeben
                        return sessionID;                        
                    }
                    _traceSource.TraceEvent(TraceEventType.Verbose, 2009, "Create new session.");

                    // Neue Sitzung erzeugen
                    Session session = new Session(identity, DateTime.Now);

                    // Sitzungsschlssel lesen
                    sessionID = session.SessionID;

                    _traceSource.TraceEvent(TraceEventType.Verbose, 2010, "-> Session ID: {0}",sessionID.ToString());

                    // Sitzung der Sitzungsliste zufgen
                    _sessions.Add(sessionID, session);

                    // Benutzername und Sitzungsschlssel zur Benutzernamen-Suchindexliste zufgen
                    _userNameIndex.Add(identity.Name, sessionID);

                    _traceSource.TraceEvent(TraceEventType.Verbose, 2006, "Logon successful.");

                    // Sitzungsschlssel zurckgeben
                    return sessionID;
                }
            }
            _traceSource.TraceEvent(TraceEventType.Error, 2305, "Logon failed.");
                
            // Leeren Sitzungsschlssel zurckgeben
            return Guid.Empty;
        }

        /// <summary>
        /// Prft, ob der Aufrufer korrekt angemeldet ist, und eine bestimmte Rolle hat, oder nicht.
        /// </summary>
        /// <param name="roleName">Rollenname</param>
        /// <returns>Prfergebnis</returns>
        public bool IsInRole(string roleName)
        {
            // Wenn der Aufrufer vertrauenswrdig ist ...
            if (IsCallerTrustworthy)            
            {
                // Rollenprfung durchfhren
                return CheckRole(roleName);                
            }  
            // Falsch zurckgeben
            return false;
        }

        /// <summary>
        /// Prft, ob der Aufrufer vertrauenswrdig ist, oder nicht.
        /// </summary>
        /// <returns>Prfergebnis</returns>
        public bool IsCallerTrustworthy
        {
            get
            {
                _traceSource.TraceEvent(TraceEventType.Information, 2105, "Is {0} trustworthy?", ApplicationServer.CurrentUser.Name);

                // Wenn der Aufrufer korrekt authentifiziert ist ...
                if (ApplicationServer.CurrentUser.IsAuthenticated)
                {
                    // Sitzungsschlssel aus dem Aufrufkontext lesen
                    Guid sessionID = ApplicationServer.SessionID;

                    // Wenn dem Aufrufkontext keine Sitzungsinformationen angehngt sind ...
                    if (sessionID == Guid.Empty)
                    {
                        _traceSource.TraceEvent(TraceEventType.Error, 2307, "-> No session in context.");

                        // Falsch zurckgeben
                        return false;
                    }

                    // Sitzungsobjekt
                    Session session = null;

                    // Wenn eine Sitzung mit dem Schlssel aus dem Aufrufkontext existiert ...
                    if (_sessions.TryGetValue(sessionID, out session))
                    {
                        // Wenn die Identitt des Aufrufers nicht mit der Identitt der bestehenden Sitzung bereinstimmt ...
                        if (session.Identity.Name != ApplicationServer.CurrentUser.Name)
                        {
                            _traceSource.TraceEvent(TraceEventType.Error, 2304, "-> Failed: Wrong identity.");

                            // Falsch zurckgeben
                            return false;
                        }
                        // Wenn die Sitzung abgelaufen ist ...
                        if (IsSessionExpired(session))
                        {
                            // Schalter fr Ergebnis der Identittsprfung beim Erneuern der Sitzung
                            bool wrongIdentity = false;

                            // Wenn die Sitzung nicht erneuert werden konnte ...
                            if (!RenewSession(session, out wrongIdentity))
                            {
                                // Wenn die Identitt stimmt ...
                                if (!wrongIdentity)
                                {
                                    // Benutzernamen Sucheintrag lschen
                                    _userNameIndex.Remove(new KeyValuePair<string, Guid>(_sessions[sessionID].Identity.Name, sessionID));

                                    // Sitzung lschen
                                    _sessions.Remove(sessionID);
                                }
                                // Falsch zurckgeben
                                return false;
                            }
                        }
                        _traceSource.TraceEvent(TraceEventType.Verbose, 2013, "-> OK");

                        // Wahr zurckgeben
                        return true;
                    }
                    _traceSource.TraceEvent(TraceEventType.Error, 2306, "Session '{0}' not found.", sessionID.ToString());
                }
                _traceSource.TraceEvent(TraceEventType.Error, 2302, "Caller is not authenticated.");

                // Falsch zurckgeben
                return false;
            }
        }

        /// <summary>
        /// Beendet eine Benutzersitzung.
        /// </summary>
        public void Logoff()
        {
            _traceSource.TraceEvent(TraceEventType.Information, 2104, "Logoff.");
            _traceSource.TraceEvent(TraceEventType.Verbose, 2015, "-> User: {0}", ApplicationServer.CurrentUser.Name);

            // Wenn der Aufrufer sich korrekt ausweisen kann ...
            if (PassportCheck())
            {
                // Ggf. bestehende Sitzung lesen
                Guid sessionID = ApplicationServer.SessionID;

                // Sitzungsliste fr den gleichzeitigen Zugriff mehrerer Threads sperren
                lock (_sessions)
                {
                    // Wenn der Benutzer angemeldet ist ...
                    if (sessionID != Guid.Empty)
                    {
                        _traceSource.TraceEvent(TraceEventType.Verbose, 2016, "-> Session ID: {0}", sessionID.ToString());

                        // Sitzungsobjekt
                        Session session=null;

                        // Wenn eine Sitzung mit diesem Schlssel existiert ... 
                        if (_sessions.TryGetValue(sessionID,out session))
                        {
                            // Wenn die Identitt des Aufrufers mit der Indentitt der bestehenden Sitzung bereinstimmt ...
                            if (session.Identity.Name == ApplicationServer.CurrentUser.Name)
                            {
                                // Benutzernamen Sucheintrag lschen
                                _userNameIndex.Remove(new KeyValuePair<string, Guid>(session.Identity.Name, sessionID));

                                // Sitzung lschen
                                _sessions.Remove(sessionID);
                            }
                        }
                        // Sitzungsinformationen aus dem Aufrufkontext entfernen
                        CallContext.FreeNamedDataSlot("session");

                        _traceSource.TraceEvent(TraceEventType.Verbose, 2015, "-> Session removed.");
                    }
                }
            }
        }

        /// <summary>
        /// Gibt die Liste der eingebauten Rollen zurck.
        /// </summary>
        public IList<string> BuiltInRoles
        {
            get
            {
                // Liste zurckgeben
                return _builtInRoles;
            }
        }

        /// <summary>
        /// Erstellt eine neue Rolle.
        /// </summary>
        /// <param name="roleName">Rollenname</param>
        /// <param name="windowsGroup">Windows-Gruppe</param>
        public void CreateRole(string roleName, string windowsGroup)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        /// <summary>
        /// Lscht eine vorhandene Rolle.
        /// </summary>
        /// <param name="roleName">Rollenname</param>
        public void DeleteRole(string roleName)
        {
            throw new Exception("The method or operation is not implemented.");
        }

        /// <summary>
        /// Gibt die Windows-Gruppe einer Rolle zurck.
        /// </summary>
        /// <param name="roleName">Rollenname</param>
        /// <returns>Windows-Gruppe</returns>
        public string GetWindowsGroupFromRole(string roleName)
        {
            // Wenn der Aufrufer korrekt authentifiziert ist ...
            if (ApplicationServer.CurrentUser.IsAuthenticated)
            {
                // Wenn der Aufrufer in der Administrator-Rolle ist ...
                if (CheckRole(ApplicationServer.ROLE_ADMINISTRATOR))
                {
                    // Wenn die angegebene Rolle existiert ...
                    if (_roleStore.ExistRole(roleName))
                        // Zugeordnete Windows-Gruppe der Rolle zurckgeben
                        return _roleStore.GetRoleByName(roleName).WindowsGroup;
                }
            }
            else
                _traceSource.TraceEvent(TraceEventType.Error, 2302, "Caller is not authenticated.");

            // Leere Zeichenfolge zurckgeben
            return string.Empty;
        }

        /// <summary>
        /// Prft, ob eine bestimmte Rolle existiert.
        /// </summary>
        /// <param name="roleName">Rollenname</param>
        /// <returns>Prfergebnis</returns>
        public bool ExistRole(string roleName)
        {
            // Wenn der Aufrufer korrekt authentifiziert ist ...
            if (ApplicationServer.CurrentUser.IsAuthenticated)
            {
                // Wahr zurckgeben, wenn eine Rolle mit dem angegebenen Namen existiert ...
                return _roleStore.ExistRole(roleName);
            }
            // Falsch zurckgeben
            return false;
        }

        /// <summary>
        /// Gibt eine Liste der Namen aller angelegten Rollen zurck.
        /// </summary>
        /// <returns>Liste mit Rollennamen</returns>
        public IList<string> GetRoleNames()
        {
            // Wenn der Aufrufer korrekt authentifiziert ist ...
            if (ApplicationServer.CurrentUser.IsAuthenticated)
            {
                // Liste erzeugen
                IList<string> names=new List<string>();

                // Alle Rollen durchlaufen
                foreach(Role role in _roleStore.Roles)
                {
                    // Rollenname in Ergebnisliste aufnehmen
                    names.Add(role.RoleName);
                }
                // Namensliste zurckgeben
                return names;
            }
            else
                _traceSource.TraceEvent(TraceEventType.Error, 2302, "Caller is not authenticated.");

            // Nichts zurckgeben
            return null;
        }

        #endregion

        #region Hilfsfunktionen (Sicherheit)

        /// <summary>
        /// Prft, ob der Aufrufer eine bestimmte Rolle hat, oder nicht.
        /// </summary>
        /// <param name="roleName">Rollenname</param>
        /// <returns>Prfergebnis</returns>
        private bool CheckRole(string roleName)
        {
            _traceSource.TraceEvent(TraceEventType.Verbose, 2003, "Check role: {0}",roleName);

            // Wenn die angegebene Rolle existiert ...
            if (_roleStore.ExistRole(roleName))
            {
                // Windows-Gruppe der Rolle abrufen
                string windowsGroup = _roleStore.GetRoleByName(roleName).WindowsGroup;

                // Rollenprfung durchfhren
                bool isInRole = ApplicationServer.CurrentSecurityPrincipal.IsInRole(windowsGroup);

                _traceSource.TraceEvent(TraceEventType.Verbose, 2004, "-> Result: {0}",isInRole.ToString());

                // Ergebnis der Rollenprfung zurckgeben
                return isInRole;
            }
            else
            {
                Debug.Assert(false, "false // Unbekannte Rolle");
                _traceSource.TraceEvent(TraceEventType.Error, 2303, "Unknown role: {0}",roleName);
            }

            // Falsch zurckgeben
            return false;
        }

        /// <summary>
        /// berprft, ob sich der Aufrufer korrekt ausweisen kann und mindestens in der Rolle "Benutzer" ist.
        /// </summary>
        /// <returns>Prfergebnis (Wahr, wenn der Aufrufer okay ist)</returns>
        private bool PassportCheck()
        {
            // Wenn der Aufrufer korrekt authentifiziert ist ...
            if (ApplicationServer.CurrentUser.IsAuthenticated)            
                // Wahr zurckgeben, wenn der Aufrufer in der Benutzer-Rolle ist ...
                return CheckRole(ApplicationServer.ROLE_USER);
            else
                _traceSource.TraceEvent(TraceEventType.Error, 2302, "Caller is not authenticated.");
                
            // Falsch zurckgeben
            return false;
        }

        /// <summary>
        /// Erneuert eine bestimmte Sitzung.
        /// </summary>
        /// <param name="sessionID">Sitzungsschlssel</param>
        /// <returns>Wahr, wenn die Erneuerung erfolgreich war, ansonsten Falsch</returns>
        private bool RenewSession(Guid sessionID)
        {
            // Schalter fr falsche Identitt
            bool wrongIdentity=false;

            // berladung aufrufen
            return RenewSession(sessionID,out wrongIdentity);
        }

        /// <summary>
        /// Erneuert eine bestimmte Sitzung.
        /// </summary>
        /// <param name="sessionID">Sitzungsschlssel</param>
        /// <param name="wrongIdentity">Gibt Wahr zurck, wenn die Identitt des Aufrufers nicht mit der Identitt der angegebenen Sitzung bereinstimmt</param>
        /// <returns>Wahr, wenn die Erneuerung erfolgreich war, ansonsten Falsch</returns>
        private bool RenewSession(Guid sessionID,out bool wrongIdentity)
        {
            // Sitzungsobjekt
            Session session = null;

            // Standardmig nicht von falscher Identitt ausgehen
            wrongIdentity=false;

            // Wenn die angegebene Sitzung existiert ...
            if (_sessions.TryGetValue(sessionID, out session))
            {                
                // berladung aufrufen
                return RenewSession(session, out wrongIdentity);
            }
            _traceSource.TraceEvent(TraceEventType.Error, 2306, "Session '{0}' not found.",sessionID.ToString());

            // Falsch zurckgeben
            return false;
        }

        /// <summary>
        /// Erneuert eine bestimmte Sitzung.
        /// </summary>
        /// <param name="session">Sitzung</param>
        /// <param name="wrongIdentity">Gibt Wahr zurck, wenn die Identitt des Aufrufers nicht mit der Identitt der angegebenen Sitzung bereinstimmt</param>
        /// <returns>Wahr, wenn die Erneuerung erfolgreich war, ansonsten Falsch</returns>
        private bool RenewSession(Session session, out bool wrongIdentity)
        {
            _traceSource.TraceEvent(TraceEventType.Information, 2103, "Renew session.");
            _traceSource.TraceEvent(TraceEventType.Verbose, 2016, "-> Session ID: {0}", session.SessionID.ToString());

            // Standardmig nicht von falscher Identitt ausgehen
            wrongIdentity = false;
            
            // Wenn die Identitt des Aufrufers mit der Identitt der bestehenden Sitzung bereinstimmt ...
            if (session.Identity.Name == ApplicationServer.CurrentUser.Name)
            {
                // Zeitstempel der Sitzung aktualisieren
                session.Renew();

                _traceSource.TraceEvent(TraceEventType.Verbose, 2017, "-> Session renewed");

                // Wahr zurckgeben
                return true;
            }
            else
            {
                _traceSource.TraceEvent(TraceEventType.Error, 2304, "-> Failed: Wrong identity.");

                // Schalter fr falsche Identitt setzen
                wrongIdentity = true;
            }
            
            // Falsch zurckgeben
            return false;
        }

        /// <summary>
        /// Gibt zurck, ob eine bestimmte Sitzung abgelaufen ist, oder nicht.
        /// </summary>
        /// <param name="sessionID">Sitzungsschlssel</param>
        /// <returns>Prfergebnis</returns>
        private bool IsSessionExpired(Guid sessionID)
        {
            // Wenn kein gltiger Sitzungsschlssel bergeben wurde ...
            if (sessionID == null || sessionID == Guid.Empty)
                // Ausnahme werfen
                throw new ArgumentNullException();

            // Maximale Sitzungslebensdauer in Sekunden aus der Konfiguration lesen
            int maxSessionLifetime = Properties.Settings.Default.MaxSessionLifetime;

            // Sitzungsobjekt
            Session session = null;

            // Wenn eine Sitzung mit diesem Sitzungsschlssel existiert ...
            if (_sessions.TryGetValue(sessionID, out session))
                // berladung aufrufen
                return IsSessionExpired(session);
            else
                // Wahr zurckgeben
                return true;
        }

        /// <summary>
        /// Gibt zurck, ob eine bestimmte Sitzung abgelaufen ist, oder nicht.
        /// </summary>
        /// <param name="session">Sitzung</param>
        /// <returns>Prfergebnis</returns>
        private bool IsSessionExpired(Session session)
        {
            // Maximale Sitzungslebensdauer in Sekunden aus der Konfiguration lesen
            int maxSessionLifetime = Properties.Settings.Default.MaxSessionLifetime;
            
            // Wahr zurckgeben, wenn die Sitzung abgelaufen ist
            return (DateTime.Now > session.Timestamp.AddMinutes(Convert.ToDouble(maxSessionLifetime)));
        }

        #endregion

        #region Persistenz des Rollenspeichers

        /// <summary>
        /// Gibt den kompletten Pfad zur Rollenspeicher-Datei zurck.
        /// </summary>
        private string RoleStoreFileName
        {
            get
            {
                // Name der Speicherdatei aus der Konfiguration lesen
                string storeFileName = Properties.Settings.Default.RoleStoreFile;

                // Wenn kein Dateiname konfiguriert ist ...
                if (string.IsNullOrEmpty(storeFileName))
                    // Ausnahme werfen
                    throw new ApplicationException(LanguageResource.InvalidRoleStoreConfig);

                // Programmverzeichnis ermitteln
                string programDir = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location);

                // Absoluter Dateiname zur Rollenspeicherdatei zusammenbauen und zurckgeben
                return Path.Combine(programDir, storeFileName);
            }
        }

        /// <summary>
        /// Ldt den Rollenspeicher von Festplatte.
        /// </summary>
        private void LoadRoleStore()
        {
            // Wenn bereits ein Rollenspeicher geladen ist ...
            if (_roleStore != null)
                // Bestehenden Rollenspeicher entsorgen
                _roleStore = null;

            try
            {
                _traceSource.TraceEvent(TraceEventType.Information, 2002, "Loading role store from '{0}'.", RoleStoreFileName); 

                // XML-Leser zum ffnen des Rollenspeichers erzeugen
                using (XmlReader reader = XmlReader.Create(RoleStoreFileName))
                {
                    // XML-Serialisierer fr den Rollenspeicher erzeugen
                    XmlSerializer serializer = new XmlSerializer(typeof(RoleStore));

                    // Rollenspeicher aus der Datei in den Arbeitsspeicher laden
                    _roleStore = (RoleStore)serializer.Deserialize(reader);

                    // Alle eingebauten Rollen durchlaufen
                    foreach (string builtInRole in _builtInRoles)
                    {
                        // Wenn die aktuelle eingebaute Rolle nicht im Rollenspeicher definiert ist ...
                        if (!_roleStore.ExistRole(builtInRole))
                            // Ausnahme werfen
                            throw new ApplicationException(string.Format(LanguageResource.BuiltInRoleMissing, builtInRole));
                    }
                }
            }
            catch (Exception ex)
            {
                // Ausnahme einpacken und weiterwerfen
                throw new ApplicationException(LanguageResource.CannotOpenRoleStore, ex);
            }
        }

        /// <summary>
        /// Speichert den Rollenspeicher auf Festplatte.
        /// </summary>
        private void SaveRoleStore()
        {
            lock (_roleStore)
            {
                try
                {
                    // XML-Schreiber zum speichern des Rollenspeichers erzeugen
                    using (XmlWriter writer = XmlWriter.Create(RoleStoreFileName))
                    {
                        // Elemente eingerckt schreiben
                        writer.Settings.Indent = true;

                        // XML-Serialisierer fr den Rollenspeicher erzeugen
                        XmlSerializer serializer = new XmlSerializer(typeof(RoleStore));

                        // Rollenspeicher in Datei speichern
                        serializer.Serialize(writer, _roleStore);
                    }
                }
                catch (Exception ex)
                {
                    // Ausnahme einpacken und weiterwerfen
                    throw new ApplicationException(LanguageResource.CannotOpenRoleStore, ex);
                }
            }
        }

        #endregion

        #region Steuerung der Lebenszeit

        /// <summary>
        /// Ruft ein Lebensdauerdienstobjekt ab, mit dem die Lebensdauerrichtlinien fr
        ///  diese Instanz gesteuert werden knnen.
        /// </summary>
        /// <returns>Lebensdauerdienstobjekt</returns>
        public override object InitializeLifetimeService()
        {
            // Nichts zurckgeben, damit der Dienst ewig lebt.
            return null;
        }

        #endregion        
    }
}
