using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.Common;
using System.Transactions;
using Rainbird.AppServer.API;
using Rainbird.Examples.NTier.ArticleMaster.Contracts;
using System.Configuration;

namespace Rainbird.Examples.NTier.ArticleMaster.BusinessLogic
{
    /// <summary>
    /// Kapselt die Haupt-Geschftslogik zur Verwaltung von Produkten.
    /// </summary>
    public class ProductManager
    {
        // Datenanbieter-Fabrik
        private static DbProviderFactory _dbFactory = null;

        // Verbindungszeoichenfolge zur Datenbank
        private static string _connectionString = string.Empty;

        /// <summary>
        /// Statischer konstruktor.
        /// </summary>
        static ProductManager()
        {
            // Verbindungszeichenfolge aus der Konfiguration lesen
            _connectionString = ConfigurationManager.AppSettings["ConnectionString"];

            // Datenanbieter-Fabrik fr SQL Server erzeugen
            _dbFactory = DbProviderFactories.GetFactory(ConfigurationManager.AppSettings["DBProvider"]);
        }

        /// <summary>
        /// Gibt ein bestimmtes Produkt anhand seines Produktschlssels zurck.
        /// </summary>
        /// <param name="productID">Eindeutiger Produktschlssel</param>
        /// <returns>Tabelle mit Produktdatensatz</returns>
        public ArticleMasterDataSet.ProductsDataTable GetProduct(Guid productID)
        { 
            // Parameterliste erstellen
            IList<DbParameter> parameters=new List<DbParameter>();
            parameters.Add(DataAccess.CreateParameter(ProductManager._dbFactory,"@productID",DbType.Guid,ParameterDirection.Input,productID));
    
            // Tabelle fr Ergebnisrckgabe erzeugen
            ArticleMasterDataSet.ProductsDataTable result=new ArticleMasterDataSet.ProductsDataTable();

            // SQL-Code
            string sql="SELECT * FROM Products WHERE ProductID=@productID";

            // Datenbank abfragen
            DataAccess.Query(ProductManager._dbFactory,result,ProductManager._connectionString,sql,parameters);

            // Abfrageergebnisse zurckgeben
            return result;
        }

        /// <summary>
        /// Gibt ein bestimmtes Produkt anhand seiner Produktnummer zurck.
        /// </summary>
        /// <param name="productNumber">Produktnummer</param>
        /// <returns>Tabelle mit Produktdatensatz</returns>
        public ArticleMasterDataSet.ProductsDataTable GetProduct(string productNumber)
        { 
            // Parameterliste erstellen
            IList<DbParameter> parameters=new List<DbParameter>();
            parameters.Add(DataAccess.CreateParameter(ProductManager._dbFactory,"@productNumber",DbType.String,30,ParameterDirection.Input,productNumber));
    
            // Tabelle fr Ergebnisrckgabe erzeugen
            ArticleMasterDataSet.ProductsDataTable result=new ArticleMasterDataSet.ProductsDataTable();

            // SQL-Code
            string sql="SELECT * FROM Products WHERE ProductNumber=@productNumber";

            // Datenbank abfragen
            DataAccess.Query(ProductManager._dbFactory,result,ProductManager._connectionString,sql,parameters);

            // Abfrageergebnisse zurckgeben
            return result;
        }

        /// <summary>
        /// Sucht Produkte anhand der Produktnummer oder eines Teils davon.
        /// </summary>
        /// <param name="pattern">Suchmuster (% = Joker)</param>
        /// <returns>Trefferliste
        /// <remarks>
        /// Enthlt folgende Spalten:
        /// 
        /// ProductID
        /// ProductNumber
        /// ProductName
        /// </remarks>
        /// </returns>
        public DataTable FindProductsByNumber(string pattern)
        { 
            // Wenn das Suchmuster kein Jokerzeichen enthlt ...
            if (!pattern.Contains("%"))
                // Jokerzeichen am Ende anfgen
                pattern += "%";

            // Parameterliste erstellen
            IList<DbParameter> parameters=new List<DbParameter>();
            parameters.Add(DataAccess.CreateParameter(ProductManager._dbFactory,"@pattern",DbType.String,30,ParameterDirection.Input,pattern));
            
            // SQL-Code
            string sql="SELECT ProductID,ProductNumber,ProductName FROM Products WHERE ProductNumber LIKE @pattern";

            // Datenbank abfragen und Trefferliste zurckgeben
            return DataAccess.Query(ProductManager._dbFactory, null, ProductManager._connectionString, sql, parameters);
        }

        /// <summary>
        /// Sucht Produkte anhand des Produktnamens oder eines Teils davon.
        /// </summary>
        /// <param name="pattern">Suchmuster (% = Joker)</param>
        /// <returns>Trefferliste
        /// <remarks>
        /// Enthlt folgende Spalten:
        /// 
        /// ProductID
        /// ProductNumber
        /// ProductName
        /// </remarks>
        /// </returns>
        public DataTable FindProductsByName(string pattern)
        {
            // Wenn das Suchmuster kein Jokerzeichen enthlt ...
            if (!pattern.Contains("%"))
                // Jokerzeichen am Ende anfgen
                pattern += "%";

            // Parameterliste erstellen
            IList<DbParameter> parameters = new List<DbParameter>();
            parameters.Add(DataAccess.CreateParameter(ProductManager._dbFactory, "@pattern", DbType.String, 30, ParameterDirection.Input, pattern));

            // SQL-Code
            string sql = "SELECT ProductID,ProductNumber,ProductName FROM Products WHERE ProductName LIKE @pattern";

            // Datenbank abfragen und Trefferliste zurckgeben
            return DataAccess.Query(ProductManager._dbFactory, null, ProductManager._connectionString, sql, parameters);
        }

        /// <summary>
        /// Speichert Produkt-Datenstze innerhlb einer Transaktion.
        /// </summary>
        /// <param name="products">Produkt-Datenstze</param>
        /// <param name="transaction">Transaktion (Optional)</param>
        /// <returns>Tabelle mit gespeicherten Produkt-Datenstzen</returns>
        public ArticleMasterDataSet.ProductsDataTable SaveProducts(ArticleMasterDataSet.ProductsDataTable products, Transaction transaction)
        {
            // Ggf. an Transaktion des Aufrufers teilnehmen
            Transaction.Current = transaction;

            // Automatische Transaktion erzwingen
            using (TransactionScope scope = new TransactionScope(TransactionScopeOption.Required))
            {
                // Alle Produkte durchlaufen
                foreach (ArticleMasterDataSet.ProductsRow product in products)
                { 
                    // Zustand auswerten
                    switch (product.RowState)
                    { 
                        case DataRowState.Added: // Neues Produkt

                            // Wenn kein Schlssel eingetragen ist ...
                            if (product.ProductID == Guid.Empty)
                                // Produktschlssel vergeben
                                product.ProductID = Guid.NewGuid();
                            
                            // Ersteller und Erstellungszeitpunkt eintragen
                            product.CreatedBy = ApplicationServer.CurrentUser.Name;
                            product.CreatedOn = DateTime.Now;

                            break;
                        case DataRowState.Modified: // Gendertes Produkt

                            // nderer und nderungsdatum eintragen
                            product.ChangedBy = ApplicationServer.CurrentUser.Name;
                            product.ChangedOn = DateTime.Now;

                            break;
                    }
                }
                // nderungen persistieren
                DataAccess.Update(ProductManager._dbFactory, ProductManager._connectionString, products);

                // Transaktion(steil) erfolgreich abschlieen
                scope.Complete();
            }
            // Gespeicherte Tabelle zurckgeben
            return products;
        }
    }
}
