using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Rainbird.AppServer.API;
using Rainbird.Examples.NTier.ArticleMaster.Contracts;

namespace Rainbird.Examples.NTier.ArticleMaster.WindowsGUI
{
    /// <summary>
    /// Formular zur Suche von Produkten.
    /// </summary>
    public partial class FindProductsForm : Form
    {
        #region Deklarationen

        /// <summary>
        /// Ereignis: Aktion - Produkt ffnen.
        /// </summary>
        public event EventHandler<ProductEventArgs> ActionOpenProduct;

        /// <summary>
        /// Ereignis Aktion - Neues Produkt.
        /// </summary>
        public event EventHandler ActionNewProduct;

        // Stellvertreterobjekt fr den Zugriff auf den Geschftsdienst "Artikelstammverwaltung"
        private IArticleMasterService _articleMasterServiceProxy = null;

        #endregion

        #region Singleton-Implementierung

        // Singleton-Formularinstanz
        private static FindProductsForm _singleton = null;

        /// <summary>
        /// Gibt die Singleton-Instanz des Suchformulars zurck.
        /// </summary>
        public static FindProductsForm Instance
        {
            get
            { 
                // Wenn noch keine Singleton-Instanz existiert ...
                if (_singleton == null)
                    // Instanz erzeugen
                    _singleton = new FindProductsForm();
                else if (_singleton.IsDisposed) // Wenn die Instanz bereits entsorgt wurde ...
                    // Instanz erzeugen
                    _singleton = new FindProductsForm();

                // Singleton-Instanz zurckgeben
                return _singleton;
            }
        }

        #endregion

        #region Konstruktoren

        /// <summary>
        /// Privater Standarskonstruktor.
        /// </summary>
        private FindProductsForm()
        {
            // Steuerelemente vorbereiten
            InitializeComponent();

            // Automatisches Generieren von Spalten fr das Tefferlisten-Datengritter abschalten
            _resultGrid.AutoGenerateColumns = false;
        }

        #endregion

        #region Hilfsfunktionen

        /// <summary>
        /// Gibt das Stellvertreterobjekt fr den Zugriff auf die Artikelstammverwaltung zurck
        /// und stellt vorher sich, dass eine Verbindung hergestellt ist.
        /// </summary>
        private IArticleMasterService ArticleMasterServiceProxy
        {
            get
            {
                // Wenn noch kein Stellvertreterobjekt erzeugt wurde ...
                if (_articleMasterServiceProxy == null)
                    // Verbindung zur Artikelstammverwaltung herstellen
                    _articleMasterServiceProxy = ServiceFactory<IArticleMasterService>.CreateProxy();

                // Stellvertreterobjekt zurckgeben
                return _articleMasterServiceProxy;
            }
        }

        #endregion

        #region Produktsuche

        /// <summary>
        /// Sucht Produkte anhand ihrer Artikelnummer.
        /// </summary>
        private void FindProductsByProductNumber()
        {
            // Sanduhr anzeigen
            Cursor = Cursors.WaitCursor;

            // Produkte suchen und Trefferliste anzeigen
            _binding.DataSource = ArticleMasterServiceProxy.FindProductsByNumber(_findByProductNumberBox.Text);

            // Mauspfeil anzeigen
            Cursor = Cursors.Default;
        }

        /// <summary>
        /// Sucht Produkte anhand ihres Produktnamens.
        /// </summary>
        private void FindProductsByProductName()
        {
            // Sanduhr anzeigen
            Cursor = Cursors.WaitCursor;

            // Produkte suchen und Trefferliste anzeigen
            _binding.DataSource = ArticleMasterServiceProxy.FindProductsByName(_findByProductNameBox.Text);

            // Mauspfeil anzeigen
            Cursor = Cursors.Default;
        }

        /// <summary>
        /// ffnet das ausgewhlte Produkt.
        /// </summary>
        private void OpenProduct()
        { 
            // Wenn ein Produkt ausgewhlt ist ...
            if (_binding.Current != null)
            {
                // Produktschlssel des ausgewhlten Produkts ermitteln
                Guid productID = (Guid)((DataRowView)_binding.Current)["ProductID"];

                // Wenn fr ActionOpenProduct Ereignisprozeduren registriert sind ...
                if (ActionOpenProduct != null)
                    // Ereignis feuern
                    ActionOpenProduct(this, new ProductEventArgs(productID));
            }
        }

        #endregion

        #region Formular-Ereignisprozeduren

        /// <summary>
        /// Wird ausgefhrt, wenn der Artikelnummern-Suchknopf geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _findByProductNumberButton_Click(object sender, EventArgs e)
        {
            // Suche starten
            FindProductsByProductNumber();
        }

        /// <summary>
        /// Wird ausgefhrt, wenn der Produktnamen-Suchknopf geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _findByProductNameButton_Click(object sender, EventArgs e)
        {
            // Suche starten
            FindProductsByProductName();
        }

        /// <summary>
        /// Wird ausgefhrt, wenn das Artikelnummern-Suchfeld den Fokus erhlt.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _findByProductNumberBox_Enter(object sender, EventArgs e)
        {
            // Andere Suchfelder leeren
            _findByProductNameBox.Text = string.Empty;
        }

        /// <summary>
        /// Wird ausgefhrt, wenn das Produktnamen-Suchfeld den Fokus erhlt.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _findByProductNameBox_Enter(object sender, EventArgs e)
        {
            // Andere Suchfelder leeren
            _findByProductNumberBox.Text = string.Empty;
        }
        
        /// <summary>
        /// Wird ausgefhrt, wenn im Artikelnummern-Suchfeld eine Taste runtergedrckt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _findByProductNumberBox_KeyDown(object sender, KeyEventArgs e)
        {
            // Wenn Enter gedrckt wurde ...
            if (e.KeyCode == Keys.Enter)
            {
                // Tastenbefehl als verarbeitet markien
                e.SuppressKeyPress = true;

                // Suche starten
                FindProductsByProductNumber();
            }
        }

        /// <summary>
        /// Wird ausgefhrt, wenn im Produktnamen-Suchfeld eine Taste runtergedrckt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _findByProductNameBox_KeyDown(object sender, KeyEventArgs e)
        {
            // Wenn Enter gedrckt wurde ...
            if (e.KeyCode == Keys.Enter)
            {
                // Tastenbefehl als verarbeitet markien
                e.SuppressKeyPress = true;
                
                // Suche starten
                FindProductsByProductName();
            }
        }

        /// <summary>
        /// Wird ausgefhrt, wenn auf den Symbolleistenknopf "Neu" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _createProductTool_Click(object sender, EventArgs e)
        {
            // Wenn fr ActionNewProduct Ereignisprozeduren registriert sind ...
            if (ActionNewProduct != null)
                // Ereignis feuern
                ActionNewProduct(this, null);
        }

        /// <summary>
        /// Wird ausgefhrt, wenn auf den Symbolleistenknopf "ffnen" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _openProductTool_Click(object sender, EventArgs e)
        {
            // Ausgewhltes Produkt ffnen
            OpenProduct();
        }

        /// <summary>
        /// Wird ausgefhrt, wenn auf das Trefferlisten-Datengitter doppelgeklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _resultGrid_DoubleClick(object sender, EventArgs e)
        {
            // Ausgewhltes Produkt ffnen
            OpenProduct();
        }

        #endregion
    }
}