using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
using Rainbird.AppServer.API;
using Rainbird.Examples.NTier.ArticleMaster.Contracts;

namespace Rainbird.Examples.NTier.ArticleMaster.WindowsGUI
{
    /// <summary>
    /// Steuerelement zu Anzeige und Eingabe von Daten eines einzelnen Produkts.
    /// </summary>
    public partial class ProductEditPanel : UserControl
    {
        // Stellvertreterobjekt fr den Zugriff auf den Geschftsdienst "Artikelstammverwaltung"
        private IArticleMasterService _articleMasterServiceProxy = null;

        // Schalter fr den Bearbeistungsmodus
        private bool _editable = false;

        /// <summary>
        /// Standardkonstruktor.
        /// </summary>
        public ProductEditPanel()
        {
            // Steuerelemente vorbereiten
            InitializeComponent();

            // Die Ansicht startet nicht im Bearbeitungsmodus
            _editable = false;

            // Gebundene Steuerelemente einfrben und (ent)sperren
            ColorizeAndUnLockBoundControls();

            // Preisanzeige formatieren
            _priceBox.DataBindings[0].FormatString = "c";
        }

        /// <summary>
        /// Gibt Wahr zurck, wenn sich die Ansicht im Bearbeitungsmodus befindet, ansonsten Falsch. 
        /// </summary>
        public bool Editable
        {
            get { return _editable; }
        }

        /// <summary>
        /// Frbt alle gebundenen bearbeitbaren Steuerelemente entsprechend des Bearbeitungsmodus ein.
        /// Setzt auerdem den Schreibschutz entsprechend.
        /// </summary>
        private void ColorizeAndUnLockBoundControls()
        {
            // Variable fr Farbe
            Color color = SystemColors.Window;
            
            // Wenn sich die Ansicht im Bearbeitungsmodus befindet ...
            if (_editable)
                // Farbe auf Hellgelb abndern
                color = Color.LightYellow;

            // Alle Steuerelement-Bindungen durchlaufen
            foreach (Binding binding in _binding.CurrencyManager.Bindings)
            {
                // Typ des Steuerelements auswerten
                switch (binding.Control.GetType().Name)
                {
                    case "TextBox": // Textfeld

                        // Farbe festlegen
                        binding.Control.BackColor = color;

                        // Schreibschutz festlegen
                        ((TextBox)binding.Control).ReadOnly = !_editable;

                        break;
                    default: // Sonstige Steuerelemente

                        break;
                }
            }
        }

        /// <summary>
        /// Versetzt die Ansicht in den Bearbeitungsmodus.
        /// </summary>
        public void Edit()
        {
            // Wenn sich die Ansicht noch nicht im Bearbeitungsmodus befindet ...
            if (!_editable)
            {  
                // Wenn ein Produkt geladen ist ...
                if (ProductID != Guid.Empty)
                {
                    // Versuchen, eine Sperre auf das Produkt zu bekommen 
                    LockingInfo lockInfo = ApplicationServer.AcquireLock("Products", ProductID.ToString());

                    // Wenn die Sperre zugeteilt wurde ...
                    if (lockInfo.Success)
                    {
                        // Bearbeitungsmodus aktivieren
                        _editable = true;

                        // Gebundene Steuerelemente einfrben und (ent)sperren
                        ColorizeAndUnLockBoundControls();
                    }
                    else
                    {
                        // Warnmeldung anzeigen
                        MessageBox.Show(lockInfo.Message, "Rainbirds n-Tier Example", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
            }
        }

        /// <summary>
        /// Gibt das Stellvertreterobjekt fr den Zugriff auf die Artikelstammverwaltung zurck
        /// und stellt vorher sicher, dass eine Verbindung hergestellt ist.
        /// </summary>
        private IArticleMasterService ArticleMasterServiceProxy
        {
            get
            { 
                // Wenn noch kein Stellvertreterobjekt erzeugt wurde ...
                if (_articleMasterServiceProxy==null)
                    // Verbindung zur Artikelstammverwaltung herstellen
                    _articleMasterServiceProxy = ServiceFactory<IArticleMasterService>.CreateProxy();

                // Stellvertreterobjekt zurckgeben
                return _articleMasterServiceProxy;
            }
        }

        /// <summary>
        /// Gibt Schlssel des geladenen Produkts zurck, oder legt ihn fest.
        /// </summary>
        public Guid ProductID
        {
            get
            { 
                // Wenn ein Produkt geladen ist ...
                if (_productData.Products.Count == 1)
                    // Produktschlssel zurckgeben
                    return _productData.Products[0].ProductID;
                else
                    // Leeren Schlssel zurckgeben
                    return Guid.Empty;
            }
            set
            {
                // Ggf. geladenes Produkt entsorgen
                _productData.Products.Rows.Clear();

                // Wenn ein gltiger Schlssel bergeben wurde ...
                if (value != Guid.Empty)
                {
                    // Produktdatensatz abrufen
                    ArticleMasterDataSet.ProductsDataTable products = ArticleMasterServiceProxy.GetProduct(value);

                    // Produktdatensatz lokal zwischenspeichern
                    _productData.Products.Merge(products, false);
                }
            }
        }

        /// <summary>
        /// Erzeugt ein neues Produkt.
        /// </summary>
        public void CreateNewProduct()
        {
            // Wenn sich die Ansicht nicht im Bearbeitungsmodus befindet ...
            if (!_editable)
            {
                // Ggf. Bearbeitung abschlieen
                _binding.EndEdit();

                // Ggf. geladenes Produkt entsorgen
                _productData.Products.Rows.Clear();

                // Produktdatensatz erzeugen
                ArticleMasterDataSet.ProductsRow product = _productData.Products.NewProductsRow();

                // Produktschlssel vergeben
                product.ProductID = Guid.NewGuid();

                // Datensatz an Tabelle anfgen
                _productData.Products.AddProductsRow(product);
                
                // Versuchen, eine Sperre auf das Produkt zu bekommen 
                LockingInfo lockInfo = ApplicationServer.AcquireLock("Products", ProductID.ToString());

                // Wenn die Sperre zugeteilt wurde ...
                if (lockInfo.Success)
                {
                    // Bearbeitungsmodus aktivieren
                    _editable = true;

                    // Gebundene Steuerelemente einfrben und (ent)sperren
                    ColorizeAndUnLockBoundControls();
                }
                else
                {
                    // Ggf. geladenes Produkt entsorgen
                    _productData.Products.Rows.Clear();

                    // Warnmeldung anzeigen
                    MessageBox.Show(lockInfo.Message, "Rainbirds n-Tier Example", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }

        /// <summary>
        /// Speichert nderungen am geladenen Produkt.
        /// </summary>
        public void Save()
        { 
            // Wenn ein Produkt geladen ist und sich die Ansicht im Bearbeitungsmodus befindet ...
            if ((ProductID != Guid.Empty) && _editable)
            {
                try
                {
                    // Ggf. Bearbeitung abschlieen
                    _binding.EndEdit();

                    // anderungen an den Geschftsdienst zum Validieren und Speichern bergeben
                    ArticleMasterDataSet.ProductsDataTable products=ArticleMasterServiceProxy.SaveProducts(_productData.Products, null);

                    // Zwischengespeicherte Version aktualisieren
                    _productData.Products.Merge(products, false);

                    // Bearbeitungsmodus deaktivieren
                    _editable = false;

                    // Gebundene Steuerelemente einfrben und (ent)sperren
                    ColorizeAndUnLockBoundControls();

                    // Sperre freigeben
                    ApplicationServer.ReleaseLock("Products", ProductID.ToString());
                }
                catch (Exception ex)
                { 
                    // Fehlermeldung ausgeben
                    MessageBox.Show(ex.Message);
                }
            }
        }

        /// <summary>
        /// Verwirft nderungen am geladenen Produkt.
        /// </summary>
        public void RejectChanges()
        {
            // Wenn ein Produkt geladen ist und sich die Ansicht im Bearbeitungsmodus befindet ...
            if ((ProductID != Guid.Empty) && _editable)
            {
                // Ggf. Bearbeitung abschlieen
                _binding.EndEdit();

                // nderungen verwerfen
                _productData.Products.RejectChanges();

                // Bearbeitungsmodus deaktivieren
                _editable = false;

                // Gebundene Steuerelemente einfrben und (ent)sperren
                ColorizeAndUnLockBoundControls();

                // Sperre freigeben
                ApplicationServer.ReleaseLock("Products", ProductID.ToString());
            }
        }
    }
}