using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Globalization;
using System.Threading;
using System.Windows.Forms;
using Rainbird.AppServer.API;
using Rainbird.Examples.NTier.ArticleMaster.Contracts;
using Rainbird.Examples.NTier.ArticleMaster.WindowsGUI;
using Rainbird.Examples.NTier.Inventory.Contracts;
using Rainbird.Examples.NTier.Inventory.WindowsGUI;

namespace Rainbird.Examples.NTier.ClientApplication
{
    /// <summary>
    /// Hauptfenster der Anwendung.
    /// </summary>
    public partial class ApplicationForm : Form
    {
        #region Singleton Implementierung

        // Singleton-Instanz
        private static ApplicationForm _singleton = null;

        /// <summary>
        /// Gibt die Singleton-Instanz des Anwendungsfensters zurck.
        /// </summary>
        public static ApplicationForm Instance
        {
            get
            {
                // Wenn noch keine Singleton-Instanz existiert ...
                if (_singleton == null)
                    // Instanz erzeugen
                    _singleton = new ApplicationForm();
                else if (_singleton.IsDisposed) // Wenn die Instanz bereits entsorgt wurde ...
                    // Instanz erzeugen
                    _singleton = new ApplicationForm();

                // Singleton-Instanz zurckgeben
                return _singleton;
            }
        }

        #endregion

        #region Konstruktoren

        /// <summary>
        /// Privater Standardkonstruktor.
        /// </summary>
        private ApplicationForm()
        {
            // Steuerelemente vorbereiten
            InitializeComponent();
        }

        #endregion

        #region Funktionen fr Modul Artikelstamm

        /// <summary>
        /// Schalter, um festzustellen, ob Aktionen des Produktsuche-Dialogs bereits abonniert wurden, oder nicht.
        /// </summary>
        private bool _productFindActionsRegistered = false;

        /// <summary>
        /// Zeigt das Formular zur Suche von Produkten als MDI-Unterformular an.
        /// </summary>
        private void ShowFindProductForm()
        {
            // Wenn die Produktsuche noch nicht verwendet wurde ...
            if (!_productFindActionsRegistered)
            {
                // Suchformular fr Produkte als MDI-Unterformular anzeigen
                FindProductsForm.Instance.MdiParent = this;
                FindProductsForm.Instance.Show();

                // Aktionen abonnieren  
                FindProductsForm.Instance.FormClosed += new FormClosedEventHandler(Instance_FormClosed);
                FindProductsForm.Instance.ActionNewProduct += new EventHandler(FindProductsForm_ActionNewProduct);
                FindProductsForm.Instance.ActionOpenProduct += new EventHandler<ProductEventArgs>(FindProductsForm_ActionOpenProduct);

                // Schalter setzen
                _productFindActionsRegistered = true;
            }
            else
            {
                // Suchformular fr Produkte als MDI-Unterformular anzeigen
                FindProductsForm.Instance.MdiParent = this;
                FindProductsForm.Instance.Show();
            }
        }

        /// <summary>
        /// Wird aufgerufen, wenn das Produktsuche-Formular geschlossen wurde.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void Instance_FormClosed(object sender, FormClosedEventArgs e)
        {
            // Schalter zurcksetzen (Damit die Aktionen der Produktsuche bei der nchsten Verwendung neu abonniert werden)
            _productFindActionsRegistered = false;   
        }

        /// <summary>
        /// Wird aufgerufen, wenn ein bestehendes Produkt geffnet werden soll.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void FindProductsForm_ActionOpenProduct(object sender, ProductEventArgs e)
        {
            // Neuen Produkteditor erzeugen
            ProductEditor editor = ProductEditor.CreateInstance();
                        
            // Editor als MDI-Unterformular anzeigen
            editor.MdiParent = this;
            editor.Show();

            // Produkt laden
            editor.LoadProduct(e.ProductID);
        }

        /// <summary>
        /// Wird aufgerufen, wenn ein neues Produkt erstellt werden soll.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void FindProductsForm_ActionNewProduct(object sender, EventArgs e)
        {
            // Neuen Produkteditor erzeugen
            ProductEditor editor = ProductEditor.CreateInstance();

            // Editor als MDI-Unterformular anzeigen
            editor.MdiParent=this;
            editor.Show();

            // Neues Produkt erzeugen
            editor.CreateNewProduct();
        }

        #endregion

        #region Funktionen fr Modul Lagerverwaltung

        /// <summary>
        /// Zeigt den Lager-Editor an.
        /// </summary>
        private void ShowRepositoryEditor()
        {
            // Lager-Editor anzeigen
            RepositoryEditor.Instance.MdiParent = this;
            RepositoryEditor.Instance.Show();
        }

        #endregion

        #region Formular-Ereignisprozeduren

        /// <summary>
        /// Wird ausgefhrt, wenn das Formular geladen wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void ApplicationForm_Load(object sender, EventArgs e)
        {
            // Wenn die aktuelle Kultureinstellung nicht in der Liste ist ...
            if (!_languageSelectionTool.Items.Contains(CultureInfo.CurrentUICulture.Name))
                // Kultureinstellung aufnehmen
                _languageSelectionTool.Items.Add(CultureInfo.CurrentUICulture.Name);

            // Aktuelle Kultureinstellung anzeigen 
            _languageSelectionTool.Text = CultureInfo.CurrentUICulture.Name;

            // Ereignis abonnieren
            _languageSelectionTool.SelectedIndexChanged += new System.EventHandler(this._languageSelectionTool_SelectedIndexChanged);
        }

        /// <summary>
        /// Wird ausgefhrt, wenn der Symbolleistenbefehl "Produkte" angeklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _articleMasterTool_Click(object sender, EventArgs e)
        {
            // Formular zur Produktsuche anzeigen
            ShowFindProductForm();
        }

        /// <summary>
        /// Wird ausgefhrt, wenn der Menbefehl "Produkte" angeklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _articleMasterModulesFileMenu_Click(object sender, EventArgs e)
        {
            // Formular zur Produktsuche anzeigen
            ShowFindProductForm();
        }

        /// <summary>
        /// Wird ausgefhrt, wenn der Menbefehl "Lager" angeklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _repositoriesFileMenu_Click(object sender, EventArgs e)
        {
            // Lager-Editor anzeigen
            ShowRepositoryEditor();
        }

        /// <summary>
        /// Wird ausgefhrt, wenn der Symbolleistenbefehl "Lager" angeklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _inventoryTool_Click(object sender, EventArgs e)
        {
            // Lager-Editor anzeigen
            ShowRepositoryEditor();
        }

        /// <summary>
        /// Wird ausgefhrt, wenn eine Kultureinstellung ausgewhlt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _languageSelectionTool_SelectedIndexChanged(object sender, EventArgs e)
        {
            // Ressourcenmanager fr das Formular erzeugen
            ComponentResourceManager resources=new ComponentResourceManager(typeof(ApplicationForm));

            // Kultureinstellung ndern
            Thread.CurrentThread.CurrentUICulture = new CultureInfo(_languageSelectionTool.Text, true);

            // Ressourcen neu anwenden
            resources.ApplyResources(this._mainMenu, "_mainMenu");
            resources.ApplyResources(this._fileMenu, "_fileMenu");
            resources.ApplyResources(this._articleMasterModulesFileMenu, "_articleMasterModulesFileMenu");
            resources.ApplyResources(this._repositoriesFileMenu, "_repositoriesFileMenu");
            resources.ApplyResources(this._quitFileMenu, "_quitFileMenu");
            resources.ApplyResources(this.toolStrip1, "toolStrip1");
            resources.ApplyResources(this._articleMasterTool, "_articleMasterTool");
            resources.ApplyResources(this.toolStripSeparator1, "toolStripSeparator1");
            resources.ApplyResources(this._inventoryTool, "_inventoryTool");
            resources.ApplyResources(this.toolStripSeparator3, "toolStripSeparator3");
            resources.ApplyResources(this.toolStripLabel1, "toolStripLabel1");
            resources.ApplyResources(this._languageSelectionTool, "_languageSelectionTool");
            resources.ApplyResources(this, "$this");
        }
       
        #endregion    
    }
}