using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Rainbird.AppServer.API;
using Rainbird.Examples.NTier.ArticleMaster.Contracts;
using Rainbird.Examples.NTier.ArticleMaster.WindowsGUI;

namespace Rainbird.Examples.NTier.ClientApplication
{
    /// <summary>
    /// Formular zum Anzeigen und Bearbeiten von Produkten und 
    /// anhngigen Daten.
    /// </summary>
    public partial class ProductEditor : Form
    {
        // Instanzliste
        private static IList<ProductEditor> _instances = null;

        /// <summary>
        /// Erstellt eine neue Instanz des Produkteditors.
        /// </summary>
        /// <returns>Formularinstanz</returns>
        public static ProductEditor CreateInstance()
        { 
            // Wenn noch keine Formularliste existiert ...
            if (_instances == null)
                // Formularliste erzeugen
                _instances = new List<ProductEditor>();

            // Neue Instanz erzeugen
            ProductEditor instance = new ProductEditor();

            // Instanz zur Instanzliste zufgen
            _instances.Add(instance);

            // Instanz zurckgeben
            return instance;
        }

        /// <summary>
        /// Gibt die Anzahl der laufenden Instanzen zurck.
        /// </summary>
        public static int InstanceCount
        {
            get
            { 
                // Wenn eine Instanzliste existiert ...
                if (_instances != null)
                    // Anzahl der Instanzen zurckgeben
                    return _instances.Count;
                else
                    // 0 zurckgeben
                    return 0;
            }
        }

        /// <summary>
        /// Wird beim schlieen des Formulars aufgerufen.
        /// </summary>
        /// <param name="e">Ereignisargumente</param>
        protected override void OnClosed(EventArgs e)
        {
            // Instanz aus der Instanzliste entfernen
            ProductEditor._instances.Remove(this);

            // OnClosed-Methode der Basisklasse aufrufen
            base.OnClosed(e);
        }

        /// <summary>
        /// Provater Standardkonstruktor.
        /// </summary>
        private ProductEditor()
        {
            // Steuerelemente vorbereiten
            InitializeComponent();

            // Symbolleistenknpfe aktualisieren
            RefreshToolbarButtons();
        }

        /// <summary>
        /// Aktualisiert die Symbolleistenknpfe entsprechend des aktuellen Status.
        /// </summary>
        private void RefreshToolbarButtons()
        {
            // Knpfe (de)aktivieren
            _createProductTool.Enabled = !_productView.Editable;
            _editTool.Enabled = !_productView.Editable && _productView.ProductID!=Guid.Empty;
            _rejectChangesTool.Enabled = _productView.Editable;
            _saveTool.Enabled = _productView.Editable;
        }

        /// <summary>
        /// Ldt ein bestimmtes Produkt.
        /// </summary>
        /// <param name="productID">Produktschlsel</param>
        public void LoadProduct(Guid productID)
        { 
            // Wenn sich die Produktansicht im nderungsmodus befindet ...
            if (_productView.Editable)
                // Hinweismeldung anzeigen
                MessageBox.Show(LanguageResource.ViewInEditMode, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            else
            {
                // Produkt laden
                _productView.ProductID = productID;
                
                // Lagerbestnde des Produkts auflisten
                _stockList.ProductID = productID;

                // Bestandsliste entsperren
                _stockList.Enabled = true;

                // Symbolleistenknpfe aktualisieren
                RefreshToolbarButtons();
            }
        }

        /// <summary>
        /// Erstellt ein neues Produkt.
        /// </summary>
        public void CreateNewProduct()
        {
            // Wenn sich die Produktansicht im nderungsmodus befindet ...
            if (_productView.Editable)
                // Hinweismeldung anzeigen
                MessageBox.Show(LanguageResource.ViewInEditMode, this.Text, MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            else
            {
                // Neues Produkt erstellen
                _productView.CreateNewProduct();

                // Bestandsliste leeren
                _stockList.ProductID = Guid.Empty;

                // Bestandsliste sperren
                _stockList.Enabled = false;

                // Symbolleistenknpfe aktualisieren
                RefreshToolbarButtons();
            }
        }

        /// <summary>
        /// Wird ausgefhrt, wenn auf den Symbolleistenbefehl "Neu" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkubftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _createProductTool_Click(object sender, EventArgs e)
        {
            // Neues Produkt erstellen
            CreateNewProduct();
        }

        /// <summary>
        /// Wird ausgefhrt, wenn auf den Symbolleistenbefehl "Bearbeiten" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkubftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _editTool_Click(object sender, EventArgs e)
        {
            // Bearbeiten
            _productView.Edit();

            // Bestandsliste sperren
            _stockList.Enabled = false;

            // Symbolleistenknpfe aktualisieren
            RefreshToolbarButtons();
        }

        /// <summary>
        /// Wird ausgefhrt, wenn auf den Symbolleistenbefehl "Verwerfen" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkubftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _rejectChangesTool_Click(object sender, EventArgs e)
        {
            // nderungen verwerfen
            _productView.RejectChanges();

            // Bestandsliste entsperren
            _stockList.Enabled = true;

            // Symbolleistenknpfe aktualisieren
            RefreshToolbarButtons();
        }

        /// <summary>
        /// Wird ausgefhrt, wenn auf den Symbolleistenbefehl "Speichern" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkubftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _saveTool_Click(object sender, EventArgs e)
        {
            // nderungen speichern
            _productView.Save();

            // Bestandsliste entsperren
            _stockList.Enabled = true;

            // Symbolleistenknpfe aktualisieren
            RefreshToolbarButtons();
        }
    }
}