using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.Common;
using System.Transactions;
using System.Diagnostics;
using Rainbird.AppServer.API;
using Rainbird.Examples.NTier.Inventory.Contracts;
using System.Configuration;

namespace Rainbird.Examples.NTier.Inventory.BusinessLogic
{
    /// <summary>
    /// Haupt-Geschftslogik zur Verwaltung der Lger.
    /// </summary>
    public class RepositoryManager
    {
        // Datenanbieter-Fabrik
        private static DbProviderFactory _dbFactory = null;

        // Verbindungszeoichenfolge zur Datenbank
        private static string _connectionString = string.Empty;

        /// <summary>
        /// Statischer konstruktor.
        /// </summary>
        static RepositoryManager()
        {
            // Verbindungszeichenfolge aus der Konfiguration lesen
            _connectionString = ConfigurationManager.AppSettings["ConnectionString"];

            Debug.Assert(!string.IsNullOrEmpty(_connectionString), "!string.IsNullOrEmpty(_connectionString)");

            // Datenanbieter aus der Konfiguration lesen
            string provider = ConfigurationManager.AppSettings["DBProvider"];

            Debug.Assert(!string.IsNullOrEmpty(provider), "!string.IsNullOrEmpty(provider)");
            
            // Datenanbieter-Fabrik fr SQL Server erzeugen
            _dbFactory = DbProviderFactories.GetFactory(ConfigurationManager.AppSettings["DBProvider"]);
        }

        /// <summary>
        /// Gibt alle angelegten Lager zurck.
        /// </summary>
        /// <returns>Alle Lager (als Tabelle)</returns>
        public InventoryDataSet.RepositoriesDataTable GetAllRepositories()
        {
            // Tabelle fr Ergebnisrckgabe erzeugen
            InventoryDataSet.RepositoriesDataTable result = new InventoryDataSet.RepositoriesDataTable();

            // SQL-Code
            string sql="SELECT * FROM Repositories ORDER BY Name";

            // Alle Lager abrufen
            DataAccess.Query(RepositoryManager._dbFactory, result, RepositoryManager._connectionString, sql, null);

            // Abfrageergebnisse zurckgeben
            return result;
        }

        /// <summary>
        /// Gibt den Namen eines bestimmten Lagers zurck.
        /// </summary>
        /// <param name="repositoryID">Lagerschlssel</param>
        /// <returns>Name</returns>
        public string GetRepositoryName(Guid repositoryID)
        { 
            // Parameterliste erzeugen
            IList<DbParameter> parameters = new List<DbParameter>();
            parameters.Add(DataAccess.CreateParameter(RepositoryManager._dbFactory, "@repositoryID", DbType.Guid, ParameterDirection.Input, repositoryID));

            // SQL-Code
            string sql="SELECT [Name] FROM Repositories WHERE RepositoryID=@repositoryID";

            // SQL-Abfrage ausfhren
            DataTable result=DataAccess.Query(RepositoryManager._dbFactory, RepositoryManager._connectionString, sql, parameters);

            // Wenn ein Lager mit dem angegebenen Schlssel gefunden wurde ...
            if (result.Rows.Count == 1)
                // Namen zurckgeben
                return (string)result.Rows[0]["Name"];
            else
                // Leere Zeichenfolge zurckgeben
                return string.Empty;
        }

        /// <summary>
        /// Sucht Lger, die keine Bestnde eines bestimmten Produkts enthalten.
        /// <remarks>
        /// Die Ergebnistabelle enthlt folgende Spalten:
        /// 
        /// RepositoryID
        /// Name
        /// </remarks>
        /// </summary>
        /// <param name="productID">Produktschlssel</param>
        /// <returns>Tabelle mit Lagerinformationen</returns>
        public DataTable FindRepositoriesWithoutProductOnStock(Guid productID)
        { 
            // Parameterliste erzeugen
            IList<DbParameter> parameters = new List<DbParameter>();
            parameters.Add(DataAccess.CreateParameter(RepositoryManager._dbFactory, "@productID", DbType.Guid, ParameterDirection.Input, productID));

            // SQL-Code
            string sql="SELECT RepositoryID,[Name] FROM Repositories WHERE RepositoryID NOT IN (SELECT DISTINCT RepositoryID FROM Stocks WHERE ProductID=@productID)";

            // SQL-Abfrage ausfhren und Ergebnisse zurckgeben
            return DataAccess.Query(RepositoryManager._dbFactory, RepositoryManager._connectionString, sql, parameters);
        }

        /// <summary>
        /// Speichert Lager-Datenstze innerhalb einer Transaktion.
        /// </summary>
        /// <param name="repositories">Lagerdatenstze</param>
        /// <param name="transaction">Transaktion (Optional)</param>
        /// <returns>Tabelle mit gespeicherten Lager-Datenstzen</returns>
        public InventoryDataSet.RepositoriesDataTable SaveRepositories(InventoryDataSet.RepositoriesDataTable repositories, Transaction transaction)
        {
            Debug.Assert(repositories != null, "repositories != null");

            // Ggf. an Transaktion des Aufrufers teilnehmen
            Transaction.Current = transaction;

            // Automatische Transaktion erzwingen
            using (TransactionScope scope = new TransactionScope(TransactionScopeOption.Required))
            {
                // Alle Lager durchlaufen
                foreach (InventoryDataSet.RepositoriesRow repository in repositories)
                {
                    // Zustand auswerten
                    switch (repository.RowState)
                    {
                        case DataRowState.Added: // Neues Lager

                            // Wenn kein Schlssel eingetragen ist ...
                            if (repository.RepositoryID == Guid.Empty)
                                // Lagerschlssel vergeben
                                repository.RepositoryID = Guid.NewGuid();

                            // Ersteller und Erstellungszeitpunkt eintragen
                            repository.CreatedBy = ApplicationServer.CurrentUser.Name;
                            repository.CreatedOn = DateTime.Now;

                            break;
                        case DataRowState.Modified: // Gendertes Lager

                            // nderer und nderungsdatum eintragen
                            repository.ChangedBy = ApplicationServer.CurrentUser.Name;
                            repository.ChangedOn = DateTime.Now;

                            break;
                    }
                }
                // nderungen persistieren
                DataAccess.Update(RepositoryManager._dbFactory, RepositoryManager._connectionString, repositories);

                // Transaktion(steil) erfolgreich abschlieen
                scope.Complete();
            }
            // Gespeicherte Tabelle zurckgeben
            return repositories;
        }
    }
}
