using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.Common;
using System.Diagnostics;
using System.Transactions;
using Rainbird.AppServer.API;
using Rainbird.Examples.NTier.Inventory.Contracts;
using System.Configuration;

namespace Rainbird.Examples.NTier.Inventory.BusinessLogic
{
    /// <summary>
    /// Haupt-Geschftslogik zur Verwaltung von Lagerbestnden.
    /// </summary>
    public class StockManager
    {
        // Datenanbieter-Fabrik
        private static DbProviderFactory _dbFactory = null;

        // Verbindungszeoichenfolge zur Datenbank
        private static string _connectionString = string.Empty;

        /// <summary>
        /// Statischer konstruktor.
        /// </summary>
        static StockManager()
        {
            // Verbindungszeichenfolge aus der Konfiguration lesen
            _connectionString = ConfigurationManager.AppSettings["ConnectionString"];

            Debug.Assert(!string.IsNullOrEmpty(_connectionString), "!string.IsNullOrEmpty(_connectionString)");

            // Datenanbieter aus der Konfiguration lesen
            string provider = ConfigurationManager.AppSettings["DBProvider"];

            Debug.Assert(!string.IsNullOrEmpty(provider), "!string.IsNullOrEmpty(provider)");

            // Datenanbieter-Fabrik fr SQL Server erzeugen
            _dbFactory = DbProviderFactories.GetFactory(ConfigurationManager.AppSettings["DBProvider"]);
        }

        /// <summary>
        /// Gibt alle Bestnde eines bestimmten Lagers zurck.
        /// </summary>
        /// <param name="repositoryID">Lagerschlssel</param>
        /// <returns>Tabelle mit Bestandsdatenstzen</returns>
        public InventoryDataSet.StocksDataTable GetStocksByRepositoryID(Guid repositoryID)
        {
            Debug.Assert(repositoryID != Guid.Empty, "repositoryID!=Guid.Empty");

            // Tabelle fr Eregbisrckgabe erzeugen
            InventoryDataSet.StocksDataTable result = new InventoryDataSet.StocksDataTable();

            // SQL-Code
            string sql = "SELECT * FROM Stocks WHERE RepositoryID=@repositoryID";

            // Parameterliste erzeugen
            IList<DbParameter> parameters = new List<DbParameter>();
            parameters.Add(DataAccess.CreateParameter(StockManager._dbFactory, "@repositoryID", DbType.Guid, ParameterDirection.Input, repositoryID));

            // Bestnde abrufen
            DataAccess.Query(StockManager._dbFactory, result, StockManager._connectionString, sql, parameters);

            // Bestandsdatenstze des angegebenen Lagers zurckgeben
            return result;
        }

        /// <summary>
        /// Gibt einen bestimmten Lagerbestand zurck.
        /// </summary>
        /// <param name="stockID">Bestandsschlssel</param>
        /// <returns>Tabelle mit Bestandsdatensatz</returns>
        public InventoryDataSet.StocksDataTable GetStock(Guid stockID)
        {
            Debug.Assert(stockID != Guid.Empty, "stockID!=Guid.Empty");

            // Tabelle fr Eregbisrckgabe erzeugen
            InventoryDataSet.StocksDataTable result = new InventoryDataSet.StocksDataTable();

            // SQL-Code
            string sql = "SELECT * FROM Stocks WHERE StockID=@stockID";

            // Parameterliste erzeugen
            IList<DbParameter> parameters = new List<DbParameter>();
            parameters.Add(DataAccess.CreateParameter(StockManager._dbFactory, "@stockID", DbType.Guid, ParameterDirection.Input, stockID));

            // Bestnde abrufen
            DataAccess.Query(StockManager._dbFactory, result, StockManager._connectionString, sql, parameters);

            // Bestandsdatensatz zurckgeben
            return result;
        }

        /// <summary>
        /// Gibt alle Lagerbestnde eines bestimmten Produkts zurck.
        /// </summary>
        /// <param name="productID">Produktschlssel</param>
        /// <returns>Tabelle mit Bestandsdatenstzen</returns>
        public InventoryDataSet.StocksDataTable GetProductStocks(Guid productID)
        {
            Debug.Assert(productID != Guid.Empty, "productID!=Guid.Empty");

            // Tabelle fr Eregbisrckgabe erzeugen
            InventoryDataSet.StocksDataTable result = new InventoryDataSet.StocksDataTable();

            // SQL-Code
            string sql = "SELECT * FROM Stocks WHERE ProductID=@productID";

            // Parameterliste erzeugen
            IList<DbParameter> parameters = new List<DbParameter>();
            parameters.Add(DataAccess.CreateParameter(StockManager._dbFactory, "@productID", DbType.Guid, ParameterDirection.Input, productID));

            // Bestnde abrufen
            DataAccess.Query(StockManager._dbFactory, result, StockManager._connectionString, sql, parameters);

            // Bestandsdatensatz zurckgeben
            return result;
        }

        /// <summary>
        /// Prft, ob ein Bestand fr ein bestimmtes Produkt bereits in einem bestimmten Lager existiert.
        /// </summary>
        /// <param name="repositoryID">Lagerschlssel</param>
        /// <param name="productID">Produktschlssel</param>
        /// <returns>Prfergebnis</returns>
        public bool ExistStock(Guid repositoryID, Guid productID)
        {
            Debug.Assert(repositoryID != Guid.Empty, "repositoryID != Guid.Empty");
            Debug.Assert(productID != Guid.Empty, "productID != Guid.Empty");

            // Parameterliste erzeugen
            IList<DbParameter> parameters = new List<DbParameter>();
            parameters.Add(DataAccess.CreateParameter(StockManager._dbFactory, "@repositoryID", DbType.Guid, ParameterDirection.Input, repositoryID));
            parameters.Add(DataAccess.CreateParameter(StockManager._dbFactory, "@productID", DbType.Guid, ParameterDirection.Input, productID));

            // SQL-Code
            string sql = "SELECT COUNT(*) AS RecordCount FROM Stocks WHERE RepositoryID=@repositoryID AND ProductID=@productID";

            // Ggf. existierenden Bestand suchen
            DataTable result = DataAccess.Query(StockManager._dbFactory, StockManager._connectionString, sql, parameters);

            // Anzahl der gefundenen Bestnde lesen
            int recordCount = (int)result.Rows[0][0];

            // Wahr zurckgeben, wenn die Anzahl nicht 0 ist, ansonsten Falsch
            return recordCount != 0;
        }

        /// <summary>
        /// Legt einen neuen Lagerbestand an.
        /// </summary>
        /// <param name="repositoryID">Lagerschlssel</param>
        /// <param name="productID">Produktschlssel</param>
        /// <param name="binLocation">Lagerort</param>
        /// <param name="estimatedStock">Soll-Bestand</param>
        /// <param name="reorderPoint">Meldebestand</param>
        /// <param name="transaction">Transaktionsobjekt (Optional)</param>
        /// <returns>Bestandsdatensatz</returns>
        public InventoryDataSet.StocksDataTable CreateStock(Guid repositoryID, Guid productID, string binLocation, decimal estimatedStock, decimal reorderPoint,Transaction transaction)
        {
            Debug.Assert(repositoryID != null, "repositoryID != null");
            Debug.Assert(productID != null, "productID != null");
            
            // Ggf. an Transaktion des Aufrufers teilnehmen
            Transaction.Current = transaction;

            // Bestandstabelle erzeugen
            InventoryDataSet.StocksDataTable stocks = new InventoryDataSet.StocksDataTable();

            // Automatische Transaktion erzwingen
            using (TransactionScope scope = new TransactionScope(TransactionScopeOption.Required))
            {
                // Wenn nicht bereits ein Bestand des angegebenen Produkts im angegebenen Lager existiert ...
                if (!ExistStock(repositoryID, productID))
                {
                    // Bestandsdatensatz erzeugen
                    InventoryDataSet.StocksRow stock = stocks.NewStocksRow();
                    stock.StockID = Guid.NewGuid(); // Eindeutigen Bestandsschlssel vergeben
                    stock.ProductID = productID;
                    stock.RepositoryID = repositoryID;
                    stock.BinLocation = binLocation;
                    stock.EstimatedStock = estimatedStock;
                    stock.ReorderPoint = reorderPoint;
                    stock.ActualStock = 0;
                    stock.CreatedBy = ApplicationServer.CurrentUser.Name;
                    stock.CreatedOn = DateTime.Now;

                    // Bestandsdatensatz an Tabelle anfgen
                    stocks.AddStocksRow(stock);

                    // nderungen persistieren
                    DataAccess.Update(StockManager._dbFactory, StockManager._connectionString, stocks);

                    // Transaktion(steil) erfolgreich abschlieen
                    scope.Complete();
                }
                else
                    // Ausnahme werfen
                    throw new ApplicationException(LanguageResource.AlreadyExistingStock);
            }
            // Tabelle mit neuem Bestandsdatensatz zurckgeben
            return stocks;
        }

        /// <summary>
        /// Aktualisiert Bestandsparameter (Meldebestand etc.).
        /// </summary>
        /// <param name="stockID">Bestandsschlssel</param>
        /// <param name="binLocation">Lagerort</param>
        /// <param name="estimatedStock">Soll-Bestand</param>
        /// <param name="reorderPoint">Meldebestand</param>
        /// <param name="transaction">Transaktionsobjekt (Optional)</param>
        /// <returns>Aktualisierter Bestandsdatensatz</returns>
        public InventoryDataSet.StocksDataTable UpdateStockParameters(Guid stockID, string binLocation, decimal estimatedStock, decimal reorderPoint, Transaction transaction)
        { 
            Debug.Assert(binLocation!=null,"binLocation!=null");
            Debug.Assert(stockID != Guid.Empty, "stockID != Guid.Empty");
            
            // Bestandstabelle erzeugen
            InventoryDataSet.StocksDataTable stocks = null;

            // Bestandsdatensatz abrufen
            stocks = GetStock(stockID);

            // Wenn der angegebene Bestand gefunden wurde ...
            if (stocks.Count == 1)
            {
                // Bestandsdatensatz adressieren
                InventoryDataSet.StocksRow stock = stocks[0];

                // Wenn sich mindestens ein angegebener Wert von den gespeicherten Werten unterscheidet ...
                if (binLocation != stock.BinLocation || estimatedStock != stock.EstimatedStock || reorderPoint != stock.ReorderPoint)
                {
                    // Ggf. an Transaktion des Aufrufers teilnehmen
                    Transaction.Current = transaction;

                    // Automatische Transaktion erzwingen
                    using (TransactionScope scope = new TransactionScope(TransactionScopeOption.Required))
                    {
                        // Bestandsparameter anpassen
                        stock.BinLocation = binLocation;
                        stock.EstimatedStock = estimatedStock;
                        stock.ReorderPoint = reorderPoint;

                        // ndrungsdatum und -Benutzer eintragen
                        stock.CreatedOn = DateTime.Now;
                        stock.ChangedBy = ApplicationServer.CurrentUser.Name;

                        // nderungen persistieren
                        DataAccess.Update(StockManager._dbFactory, StockManager._connectionString, stocks);

                        // Transaktion(steil) erfolgreich abschlieen
                        scope.Complete();
                    }   
                }               
            }
            else
                // Ausnahme werfen
                throw new ApplicationException(string.Format(LanguageResource.InvalidStockID, stockID.ToString()));

            
            // Tabelle mit aktualisiertem Bestandsdatensatz zurckgeben
            return stocks;
        }

        /// <summary>
        /// Fhrt eine Lagerbestandsbuchung durch.
        /// </summary>
        /// <param name="stockID">Bestandsschlssel</param>
        /// <param name="quantity">Menge</param>
        /// <param name="transaction">Transaktionsobjekt (Optional)</param>
        /// <returns>Neuer Ist-Bestand</returns>
        public decimal Book(Guid stockID, decimal quantity, Transaction transaction)
        {
            Debug.Assert(stockID != Guid.Empty, "stockID != Guid.Empty");
            Debug.Assert(quantity != 0, "quantity!=0");
                
            // Neuer Ist-Bestand
            decimal actualStock = 0;

            // Ggf. an Transaktion des Aufrufers teilnehmen
            Transaction.Current = transaction;

            // Automatische Transaktion erzwingen
            using (TransactionScope scope = new TransactionScope(TransactionScopeOption.Required))
            {
                // Bestandstabelle erzeugen
                InventoryDataSet.StocksDataTable stocks = null;

                // Bestandsdatensatz abrufen
                stocks = GetStock(stockID);

                // Wenn der angegebene Bestand gefunden wurde ...
                if (stocks.Count == 1)
                {
                    // Bestandsdatensatz adressieren
                    InventoryDataSet.StocksRow stock = stocks[0];
                    
                    // Buchung auf Ist-Bestand ausfhren
                    actualStock = stock.ActualStock + quantity;
                    stock.ActualStock = actualStock;

                    // ndrungsdatum und -Benutzer eintragen
                    stock.CreatedOn = DateTime.Now;
                    stock.ChangedBy = ApplicationServer.CurrentUser.Name;

                    // nderungen persistieren
                    DataAccess.Update(StockManager._dbFactory, StockManager._connectionString, stocks);

                    // Transaktion(steil) erfolgreich abschlieen
                    scope.Complete();
                }
                else
                    // Ausnahme werfen
                    throw new ApplicationException(string.Format(LanguageResource.InvalidStockID, stockID.ToString()));
            }
            // Neuen Ist-Bestand zurckgeben
            return actualStock;
        }
    }
}
