using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Security;
using System.Transactions;
using Rainbird.AppServer.API;
using Rainbird.Examples.NTier.Inventory.Contracts;
using Rainbird.Examples.NTier.Inventory.BusinessLogic;

namespace Rainbird.Examples.NTier.Inventory.Services
{
    /// <summary>
    /// Geschftsdienst "Lagerverwaltung".
    /// </summary>
    public class InventoryService : MarshalByRefObject,IInventoryService
    {
        #region IInventoryService Member

        /// <summary>
        /// Gibt alle angelegten Lager zurck.
        /// </summary>
        /// <returns>Alle Lager (als Tabelle)</returns>
        public InventoryDataSet.RepositoriesDataTable GetAllRepositories()
        {
            // Lndereinstellungen vom Aufrufer annehmen
            ApplicationServer.AssumeCultureFromSession();

            // Wenn der Aufrufer in der Rolle "Repository Reader" ist ...
            if (ApplicationServer.IsInRole("Repository Reader"))
            {
                // Aufruf an Geschftslogik delegieren
                return new RepositoryManager().GetAllRepositories();
            }
            else
                // Ausnahme werfen
                throw new SecurityException(ApplicationServer.LocalizedAccessDeniedMessage);
        }

        /// <summary>
        /// Gibt den Namen eines bestimmten Lagers zurck.
        /// </summary>
        /// <param name="repositoryID">Lagerschlssel</param>
        /// <returns>Name</returns>
        public string GetRepositoryName(Guid repositoryID)
        {
            // Lndereinstellungen vom Aufrufer annehmen
            ApplicationServer.AssumeCultureFromSession();

            // Wenn der Aufrufer in der Rolle "Repository Reader" ist ...
            if (ApplicationServer.IsInRole("Repository Reader"))
            {
                // Aufruf an Geschftslogik delegieren
                return new RepositoryManager().GetRepositoryName(repositoryID);
            }
            else
                // Ausnahme werfen
                throw new SecurityException(ApplicationServer.LocalizedAccessDeniedMessage);
        }

        /// <summary>
        /// Sucht Lger, die keine Bestnde eines bestimmten Produkts enthalten.
        /// <remarks>
        /// Die Ergebnistabelle enthlt folgende Spalten:
        /// 
        /// RepositoryID
        /// Name
        /// </remarks>
        /// </summary>
        /// <param name="productID">Produktschlssel</param>
        /// <returns>Tabelle mit Lagerinformationen</returns>
        public DataTable FindRepositoriesWithoutProductOnStock(Guid productID)
        {
            // Lndereinstellungen vom Aufrufer annehmen
            ApplicationServer.AssumeCultureFromSession();

            // Wenn der Aufrufer in der Rolle "Repository Reader" ist ...
            if (ApplicationServer.IsInRole("Repository Reader"))
            {
                // Aufruf an Geschftslogik delegieren
                return new RepositoryManager().FindRepositoriesWithoutProductOnStock(productID);
            }
            else
                // Ausnahme werfen
                throw new SecurityException(ApplicationServer.LocalizedAccessDeniedMessage);
        }

        /// <summary>
        /// Speichert Lager-Datenstze innerhalb einer Transaktion.
        /// </summary>
        /// <param name="repositories">Lagerdatenstze</param>
        /// <param name="transaction">Transaktion (Optional)</param>
        /// <returns>Tabelle mit gespeicherten Lager-Datenstzen</returns>
        public InventoryDataSet.RepositoriesDataTable SaveRepositories(InventoryDataSet.RepositoriesDataTable repositories, Transaction transaction)
        {
            // Lndereinstellungen vom Aufrufer annehmen
            ApplicationServer.AssumeCultureFromSession();

            // Wenn der Aufrufer in der Rolle "Repository Writer" ist ...
            if (ApplicationServer.IsInRole("Repository Writer"))
            {
                // Aufruf an Geschftslogik delegieren
                return new RepositoryManager().SaveRepositories(repositories, transaction);
            }
            else
                // Ausnahme werfen
                throw new SecurityException(ApplicationServer.LocalizedAccessDeniedMessage);
        }


        /// <summary>
        /// Gibt alle Bestnde eines bestimmten Lagers zurck.
        /// </summary>
        /// <param name="repositoryID">Lagerschlssel</param>
        /// <returns>Tabelle mit Bestandsdatenstzen</returns>
        public InventoryDataSet.StocksDataTable GetStocksByRepositoryID(Guid repositoryID)
        {
            // Lndereinstellungen vom Aufrufer annehmen
            ApplicationServer.AssumeCultureFromSession();

            // Wenn der Aufrufer in der Rolle "Stock Reader" ist ...
            if (ApplicationServer.IsInRole("Stock Reader"))
            {
                // Aufruf an Geschftslogik delegieren
                return new StockManager().GetStocksByRepositoryID(repositoryID);
            }
            else
                // Ausnahme werfen
                throw new SecurityException(ApplicationServer.LocalizedAccessDeniedMessage);   
        }

        /// <summary>
        /// Gibt einen bestimmten Lagerbestand zurck.
        /// </summary>
        /// <param name="stockID">Bestandsschlssel</param>
        /// <returns>Tabelle mit Bestandsdatensatz</returns>
        public InventoryDataSet.StocksDataTable GetStock(Guid stockID)
        {
            // Lndereinstellungen vom Aufrufer annehmen
            ApplicationServer.AssumeCultureFromSession();

            // Wenn der Aufrufer in der Rolle "Stock Reader" ist ...
            if (ApplicationServer.IsInRole("Stock Reader"))
            {
                // Aufruf an Geschftslogik delegieren
                return new StockManager().GetStock(stockID);
            }
            else
                // Ausnahme werfen
                throw new SecurityException(ApplicationServer.LocalizedAccessDeniedMessage);  
        }

        /// <summary>
        /// Gibt alle Lagerbestnde eines bestimmten Produkts zurck.
        /// </summary>
        /// <param name="productID">Produktschlssel</param>
        /// <returns>Tabelle mit Bestandsdatenstzen</returns>
        public InventoryDataSet.StocksDataTable GetProductStocks(Guid productID)
        {
            // Lndereinstellungen vom Aufrufer annehmen
            ApplicationServer.AssumeCultureFromSession();

            // Wenn der Aufrufer in der Rolle "Stock Reader" ist ...
            if (ApplicationServer.IsInRole("Stock Reader"))
            {
                // Aufruf an Geschftslogik delegieren
                return new StockManager().GetProductStocks(productID);
            }
            else
                // Ausnahme werfen
                throw new SecurityException(ApplicationServer.LocalizedAccessDeniedMessage);  
        }

        /// <summary>
        /// Prft, ob ein Bestand fr ein bestimmtes Produkt bereits in einem bestimmten Lager existiert.
        /// </summary>
        /// <param name="repositoryID">Lagerschlssel</param>
        /// <param name="productID">Produktschlssel</param>
        /// <returns>Prfergebnis</returns>
        public bool ExistStock(Guid repositoryID, Guid productID)
        {
            // Lndereinstellungen vom Aufrufer annehmen
            ApplicationServer.AssumeCultureFromSession();

            // Wenn der Aufrufer in der Rolle "Stock Reader" ist ...
            if (ApplicationServer.IsInRole("Stock Reader"))
            {
                // Aufruf an Geschftslogik delegieren
                return new StockManager().ExistStock(repositoryID, productID);
            }
            else
                // Ausnahme werfen
                throw new SecurityException(ApplicationServer.LocalizedAccessDeniedMessage); 
        }

        /// <summary>
        /// Legt einen neuen Lagerbestand an.
        /// </summary>
        /// <param name="repositoryID">Lagerschlssel</param>
        /// <param name="productID">Produktschlssel</param>
        /// <param name="binLocation">Lagerort</param>
        /// <param name="estimatedStock">Soll-Bestand</param>
        /// <param name="reorderPoint">Meldebestand</param>
        /// <param name="transaction">Transaktionsobjekt (Optional)</param>
        /// <returns>Bestandsdatensatz</returns>
        public InventoryDataSet.StocksDataTable CreateStock(Guid repositoryID, Guid productID, string binLocation, decimal estimatedStock, decimal reorderPoint, Transaction transaction)
        {
            // Lndereinstellungen vom Aufrufer annehmen
            ApplicationServer.AssumeCultureFromSession();

            // Wenn der Aufrufer in der Rolle "Stock Writer" ist ...
            if (ApplicationServer.IsInRole("Stock Writer"))
            {
                // Aufruf an Geschftslogik delegieren
                return new StockManager().CreateStock(repositoryID, productID, binLocation, estimatedStock, reorderPoint, transaction);
            }
            else
                // Ausnahme werfen
                throw new SecurityException(ApplicationServer.LocalizedAccessDeniedMessage);  
        }

        /// <summary>
        /// Aktualisiert Bestandsparameter (Meldebestand etc.).
        /// </summary>
        /// <param name="stockID">Bestandsschlssel</param>
        /// <param name="binLocation">Lagerort</param>
        /// <param name="estimatedStock">Soll-Bestand</param>
        /// <param name="reorderPoint">Meldebestand</param>
        /// <param name="transaction">Transaktionsobjekt (Optional)</param>
        /// <returns>Aktualisierter Bestandsdatensatz</returns>
        public InventoryDataSet.StocksDataTable UpdateStockParameters(Guid stockID, string binLocation, decimal estimatedStock, decimal reorderPoint, Transaction transaction)
        {
            // Lndereinstellungen vom Aufrufer annehmen
            ApplicationServer.AssumeCultureFromSession();

            // Wenn der Aufrufer in der Rolle "Stock Writer" ist ...
            if (ApplicationServer.IsInRole("Stock Writer"))
            {
                // Aufruf an Geschftslogik delegieren
                return new StockManager().UpdateStockParameters(stockID, binLocation, estimatedStock, reorderPoint, transaction);
            }
            else
                // Ausnahme werfen
                throw new SecurityException(ApplicationServer.LocalizedAccessDeniedMessage);
        }

        /// <summary>
        /// Fhrt eine Lagerbestandsbuchung durch.
        /// </summary>
        /// <param name="stockID">Bestandsschlssel</param>
        /// <param name="quantity">Menge</param>
        /// <param name="transaction">Transaktionsobjekt (Optional)</param>
        /// <returns>Neuer Ist-Bestand</returns>
        public decimal Book(Guid stockID, decimal quantity, Transaction transaction)
        {
            // Lndereinstellungen vom Aufrufer annehmen
            ApplicationServer.AssumeCultureFromSession();

            // Wenn der Aufrufer in der Rolle "Stock Writer" ist ...
            if (ApplicationServer.IsInRole("Stock Writer"))
            {
                // Aufruf an Geschftslogik delegieren
                return new StockManager().Book(stockID, quantity, transaction);
            }
            else
                // Ausnahme werfen
                throw new SecurityException(ApplicationServer.LocalizedAccessDeniedMessage);   
        }

        #endregion
    }
}
