using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Rainbird.AppServer.API;
using Rainbird.Examples.NTier.Inventory.Contracts;

namespace Rainbird.Examples.NTier.Inventory.WindowsGUI
{
    /// <summary>
    /// Dialog zur Erfassung einer Bestandsnderung.
    /// </summary>
    public partial class ChangeStockDialog : Form
    {
        // Objekt fr Dialogergebnisse
        private ChangeStockDialogResult _result = null;
        
        // Stellvertreterobjekt fr den Zugriff auf den Lagerdienst
        private IInventoryService _inventoryServiceProxy = null;

        /// <summary>
        /// Gibt ein Stellvertreterobjekt fr den Zugriff auf den Lagerdienst zurck.
        /// </summary>
        private IInventoryService InventoryServiceProxy
        {
            get
            { 
                // Wenn noch kein Stellvertreter existiert ...
                if (_inventoryServiceProxy == null)
                    // Stellvertreterobjekt erzeugen
                    _inventoryServiceProxy = ServiceFactory<IInventoryService>.CreateProxy();

                // Stellvertreterobjekt zurckgeben
                return _inventoryServiceProxy;
            }
        }

        /// <summary>
        /// Privater Konstruktor.
        /// </summary>
        /// <param name="result"></param>
        private ChangeStockDialog(ChangeStockDialogResult result)
        {
            // Steuerelemente vorbereiten
            InitializeComponent();

            // Objekt fr Dialogergebnisse speichern
            _result = result;

            // Aktuellen Ist-Bestand laden
            LoadActualStock();
        }

        /// <summary>
        /// Zeigt den Bestandsnderungs-Dialog fr einen bestimmten Lagerbestand an und gibt
        /// die Ergebnisse der Benutzereingaben zurck.
        /// </summary>
        /// <param name="stockID">Bestandsschlssel</param>
        /// <returns>Dialogergebnisse</returns>
        public static ChangeStockDialogResult ShowChangeStockDialog(Guid stockID)
        { 
            // Ergebnis-Objekt erzeugen
            ChangeStockDialogResult result = new ChangeStockDialogResult(stockID);

            // Dialog erzeugen
            ChangeStockDialog dialog = new ChangeStockDialog(result);

            // Dialog modal anzeigen
            dialog.ShowDialog();

            // Ergebnisse zurckgeben
            return result;
        }

        /// <summary>
        /// Ldt den Ist-Bestandswert des angegebenen Lagerbestandes.
        /// </summary>
        private void LoadActualStock()
        { 
            // Bestandsdatensatz abrufen
            InventoryDataSet.StocksDataTable stockTable = InventoryServiceProxy.GetStock(_result.StockID);

            // Wenn der Bestand nicht gefunden wurde ...
            if (stockTable.Count != 1)
                // Ausnahme werfen
                throw new ApplicationException(string.Format(LanguageResource.InvalidStockID, _result.StockID.ToString()));

            // Aktuellen Ist-Bestand im Dialog anzeigen
            _actualStockLabel.Text = stockTable[0].ActualStock.ToString();
        }

        /// <summary>
        /// Wird aufgerufen, wenn auf OK geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _cancelButton_Click(object sender, EventArgs e)
        {
            // Abbruch-Schalter setzen
            _result.Cancel = true;

            // Menge zurcksetzen
            _result.Quantity = 0m;

            // Dialog schlieen
            Close();
        }

        /// <summary>
        /// Wird aufgerufen, wenn auf Abbrechen geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _acceptButton_Click(object sender, EventArgs e)
        {
            // Sicherstellen, dass der Abbruch-Schalter nicht gesetzt ist
            _result.Cancel = false;

            // Menge bernehmen
            _result.Quantity = decimal.Parse(_quantityBox.Text);

            // Sanduhr anzeigen
            this.Cursor = Cursors.WaitCursor;

            // Lagerbuchung durchfhren
            InventoryServiceProxy.Book(_result.StockID, _result.Quantity, null);

            // Mauspfeil anzeigen
            this.Cursor = Cursors.Default;

            // Dialog schlieen
            Close();
        }

        /// <summary>
        /// Wird aufgerufen, wenn der Dialog geladen wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void ChangeStockDialog_Load(object sender, EventArgs e)
        {
            // Versuchen, eine Sperre auf den Lagerbestand zu bekommen 
            LockingInfo lockInfo = ApplicationServer.AcquireLock("Stocks", _result.StockID.ToString());

            // Wenn die Sperre nicht zugeteilt wurde ...
            if (!lockInfo.Success)
            {
                // Warnmeldung anzeigen
                MessageBox.Show(lockInfo.Message, "Rainbirds n-Tier Example", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);

                // Dialog schlieen
                Close();
            }
        }

        /// <summary>
        /// Wird aufgerufen, wenn der Dialog geschlossen wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void ChangeStockDialog_FormClosing(object sender, FormClosingEventArgs e)
        {
            // Sperre auf Lagerbestand freigeben
            ApplicationServer.ReleaseLock("Stocks", _result.StockID.ToString());            
        }
    }
}