using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Rainbird.AppServer.API;
using Rainbird.Examples.NTier.Inventory.Contracts;

namespace Rainbird.Examples.NTier.Inventory.WindowsGUI
{
    /// <summary>
    /// Lagerbestands-Ansicht fr Produkte.
    /// </summary>
    public partial class ProductStockListPanel : UserControl
    {
        // Stellvertreter-Objekt fr den Zugriff auf den Lagerverwaltungsdienst
        private IInventoryService _inventoryServiceProxy = null;

        // Produktschlssel
        private Guid _productID = Guid.Empty;

        // Tabelle mit Lagerbestands-Datenstzen
        private InventoryDataSet.StocksDataTable _stocks = null;

        /// <summary>
        /// Gibt ein Stellvertreterobjekt fr den Zugriff auf den Lagerverwaltungsdienst zurck.
        /// </summary>
        private IInventoryService InventoryServiceProxy
        {
            get
            { 
                // Wenn noch kein Stellvertreterobjekt existiert ...
                if (_inventoryServiceProxy==null)
                    // Verbindung zum Lagerverwaltungsdienst herstellen
                    _inventoryServiceProxy = ServiceFactory<IInventoryService>.CreateProxy();

                // Stellverterter-Objekt zurckgeben
                return _inventoryServiceProxy;
            }
        }

        /// <summary>
        /// Standardkonstruktor.
        /// </summary>
        public ProductStockListPanel()
        {
            // Steuerelemente vorbereiten
            InitializeComponent();
        }

        /// <summary>
        /// Gibt den Produktschlssel zurck oder legt ihn fest.
        /// </summary>
        public Guid ProductID
        {
            get { return _productID; }
            set 
            {
                // Wert bernehmen
                _productID = value;

                // Wenn nicht im Entwurfsmodus ...
                if (!this.DesignMode)
                {
                    // Wenn ein gtiger Produktschlssel angegeben wurde ...
                    if (_productID != Guid.Empty)
                    {
                        // Liste atualisieren
                        FillStockList();
                    }
                    else
                    {
                        // Lagerbestadstabelle entsorgen
                        _stocks = null;

                        // Liste leeren
                        _stockList.Items.Clear();
                    }
                }
            }
        }

        /// <summary>
        /// Gibt den Schlssel des Lagerbestandes zurck, der momentan in der Liste ausgewhlt ist, oder legt diesen fest.
        /// </summary>
        public Guid SelectedStockID
        {
            get
            { 
                // Wenn ein Bestandsschlssel in der Liste ausgewhlt ist ...
                if (_stockList.SelectedItems.Count == 1)                
                    // Bestandsschlssel aus dem Tag lesen und zurckgeben
                    return (Guid)_stockList.SelectedItems[0].Tag;
                
                // Leeren Schlssel zurckgeben
                return Guid.Empty;
            }
            set
            { 
                // Wenn ein leerer Schlssel angegeben wurde ...
                if (value == Guid.Empty)
                    // Auswahl aufheben
                    _stockList.SelectedItems.Clear();
                else
                    // Bestand in der Liste suchen
                    _stockList.Items.Find(value.ToString(),false)[0].Selected = true;
            }
        }

        /// <summary>
        /// Fllt die Lagerbestandsliste.
        /// </summary>
        private void FillStockList()
        {
            // Lagerbestnde des angegbenen Produkts abrufen
            _stocks = InventoryServiceProxy.GetProductStocks(_productID);

            // Listensteuerelement auf Aktualisierung vorbereiten
            _stockList.BeginUpdate();

            // Ggf. Liste leeren
            _stockList.Items.Clear();

            // Alle Lagerbestnde durchlaufen
            foreach (InventoryDataSet.StocksRow stock in _stocks)
            { 
                // Lagername abrufen
                string repositoryName = InventoryServiceProxy.GetRepositoryName(stock.RepositoryID);

                // Neue Zeile erzeugen
                ListViewItem item = _stockList.Items.Add(stock.StockID.ToString(),repositoryName,string.Empty);
                item.SubItems.Add(stock.BinLocation);
                item.SubItems.Add(stock.ActualStock.ToString("n"));
                item.SubItems.Add(stock.EstimatedStock.ToString("n"));
                item.SubItems.Add(stock.ReorderPoint.ToString("n"));

                // Bestandsschlssel im Tag speichern
                item.Tag = stock.StockID;
            }
            // Aktualisierung des Steuerelement abschlieen
            _stockList.EndUpdate();
        }

        /// <summary>
        /// Wird aufgerufen, wenn auf den Symbolleistenbefehl "Neuer Bestand" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _createStockTool_Click(object sender, EventArgs e)
        {
            // Wenn ein Produkt zugewiesen ist ...
            if (ProductID != Guid.Empty)
            { 
                // Dialog anzeigen
                StockEditDialog.ShowCreateStockDialog(ProductID);

                // Liste aktualisieren
                FillStockList();
            }
        }

        /// <summary>
        /// Wird aufgerufen, wenn auf den Symbolleistenbefehl "Parameter bearbeiten" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _editStockParameters_Click(object sender, EventArgs e)
        {
            // Ausgewlten Lagerschlssel abfragen
            Guid stockID = SelectedStockID;

            // Wenn ein Bestand ausgewhlt ist ...
            if (stockID != Guid.Empty)
            {
                // Dialog anzeigen
                StockEditDialog.ShowUpdateStockParametersDialog(stockID);

                // Liste aktualisieren
                FillStockList();

                // Wieder zum zuvor ausgewhlten Bestand in der Liste springen
                SelectedStockID = stockID;
            }
        }

        /// <summary>
        /// Wird aufgerufen, wenn auf den Symbolleistenbefehl "Bestandsnderung" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _changeStockTool_Click(object sender, EventArgs e)
        {
            // Ausgewlten Lagerschlssel abfragen
            Guid stockID = SelectedStockID;

            // Wenn ein Bestand ausgewhlt ist ...
            if (stockID != Guid.Empty)
            {
                // Dialog anzeigen
                ChangeStockDialogResult result = ChangeStockDialog.ShowChangeStockDialog(stockID);

                // Wenn der Benutzer nicht abgebrochen hat ...
                if (!result.Cancel)
                {
                    // Liste aktualisieren
                    FillStockList();

                    // Wieder zum zuvor ausgewhlten Bestand in der Liste springen
                    SelectedStockID = stockID;
                }
            }
        }
    }
}
