using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Diagnostics;
using System.Text;
using System.Windows.Forms;
using System.Transactions;
using Rainbird.AppServer.API;
using Rainbird.Examples.NTier.Inventory.Contracts;

namespace Rainbird.Examples.NTier.Inventory.WindowsGUI
{
    /// <summary>
    /// Formular zur Anzeige und Bearbeitung von Lagern.
    /// </summary>
    public partial class RepositoryEditor : Form
    {
        #region Deklarationen

        // Stellvertreterobjekt fr den Zugriff auf den Geschftsdienst "Lagerverwaltung"
        private IInventoryService _inventoryServiceProxy = null;

        // Schalter fr Editierbarkeit
        private bool _editable = false;

        #endregion

        #region Singleton-Implementierung

        // Singleton-Formularinstanz
        private static RepositoryEditor _singleton = null;

        /// <summary>
        /// Gibt die Singleton-Instanz des Editors zurck.
        /// </summary>
        public static RepositoryEditor Instance
        {
            get
            {
                // Wenn noch keine Singleton-Instanz existiert ...
                if (_singleton == null)
                    // Instanz erzeugen
                    _singleton = new RepositoryEditor();
                else if (_singleton.IsDisposed) // Wenn die Instanz bereits entsorgt wurde ...
                    // Instanz erzeugen
                    _singleton = new RepositoryEditor();

                // Singleton-Instanz zurckgeben
                return _singleton;
            }
        }

        #endregion

        #region Konstruktoren

        /// <summary>
        /// Standardkonstruktor.
        /// </summary>
        public RepositoryEditor()
        {
            // Steuerelemente vorbereiten
            InitializeComponent();

            // Automatische Spaltengenerierung fr das Datengitter ausschalten
            _grid.AutoGenerateColumns = false;
        }

        #endregion

        #region Hilfsfunktionen

        /// <summary>
        /// Gibt das Stellvertreterobjekt fr den Zugriff auf die Lagerverwaltung zurck
        /// und stellt vorher sich, dass eine Verbindung hergestellt ist.
        /// </summary>
        private IInventoryService InventoryServiceProxy
        {
            get
            {
                // Wenn noch kein Stellvertreterobjekt erzeugt wurde ...
                if (_inventoryServiceProxy == null)
                    // Verbindung zur Lagerverwaltung herstellen
                    _inventoryServiceProxy = ServiceFactory<IInventoryService>.CreateProxy();

                // Stellvertreterobjekt zurckgeben
                return _inventoryServiceProxy;
            }
        }

        #endregion

        #region Bearbeitungs- und Sperrlogik

        /// <summary>
        /// Gibt Wahr zurck, wenn sich die Ansicht im Bearbeitungsmodus befindet, ansonsten Falsch. 
        /// </summary>
        public bool Editable
        {
            get { return _editable; }
        }

        /// <summary>
        /// Gibt den Lagerschlssel des ausgewhlten Lagers zurck oder legt ihn fest.
        /// </summary>
        public Guid SelectedRepositoryID
        {
            get
            { 
                // Wenn ein Lager ausgewhlt ist ...
                if (_binding.Current != null)
                    // Schlssel zurckgeben
                    return (Guid)((DataRowView)_binding.Current)["RepositoryID"];
                else
                    // Leeren Schlssel zurckgeben
                    return Guid.Empty;
            }
            set
            {
                Debug.Assert(value != null, "value != null");
                Debug.Assert(value != Guid.Empty, "");

                // Wenn sich das Formular nicht im Bearbeitungsmodus befindet ...
                if (!_editable)
                    // Schlssel suchen
                    _binding.Position=_binding.Find("RepositoryID", value);
            }
        }

        /// <summary>
        /// Aktualisiert die Symbolleistenknpfe entsprechend des aktuellen Status.
        /// </summary>
        private void RefreshToolbarButtons()
        {
            // Knpfe (de)aktivieren
            _createRepositoryTool.Enabled = !_editable;
            _editTool.Enabled = !_editable && _binding.Current!=null;
            _rejectChangesTool.Enabled = _editable;
            _saveTool.Enabled = _editable;
            _firstTool.Enabled = !_editable;
            _previousTool.Enabled = !_editable;
            _indexBox.Enabled = !_editable;
            _nextTool.Enabled = !_editable;
            _lastTool.Enabled = !_editable;
        }

        /// <summary>
        /// Frbt alle gebundenen bearbeitbaren Steuerelemente entsprechend des Bearbeitungsmodus ein.
        /// Setzt auerdem den Schreibschutz entsprechend.
        /// </summary>
        private void ColorizeAndUnLockBoundControls()
        {
            // Variable fr Farbe
            Color color = SystemColors.Window;

            // Wenn sich die Ansicht im Bearbeitungsmodus befindet ...
            if (_editable)
                // Farbe auf Hellgelb abndern
                color = Color.LightYellow;

            // Alle Steuerelement-Bindungen durchlaufen
            foreach (Binding binding in _binding.CurrencyManager.Bindings)
            {
                // Typ des Steuerelements auswerten
                switch (binding.Control.GetType().Name)
                {
                    case "TextBox": // Textfeld

                        // Farbe festlegen
                        binding.Control.BackColor = color;

                        // Schreibschutz festlegen
                        ((TextBox)binding.Control).ReadOnly = !_editable;

                        break;
                    default: // Sonstige Steuerelemente

                        break;
                }
            }
        }

        /// <summary>
        /// Aktiviert den Bearbeitungsmodus.
        /// </summary>
        public void Edit()
        {
            // Wenn sich die Ansicht nicht im Bearbeitungsmodus befindet ...
            if (!_editable)
            {
                // Wenn ein Lager ausgewhlt ist ...
                if (_binding.Current != null)
                {
                    // Versuchen, eine Sperre auf das Produkt zu bekommen 
                    LockingInfo lockInfo = ApplicationServer.AcquireLock("Repositories", SelectedRepositoryID.ToString());

                    // Wenn die Sperre zugeteilt wurde ...
                    if (lockInfo.Success)
                    {
                        // Schalter fr den Bearbeitungsmodus setzen
                        _editable = true;

                        // Symbolleistenbefehle aktualisieren
                        RefreshToolbarButtons();

                        // Gebundene Steuerelemente einfrben und entsperren
                        ColorizeAndUnLockBoundControls();
                    }
                    else
                    {
                        // Warnmeldung anzeigen
                        MessageBox.Show(lockInfo.Message, "Rainbirds n-Tier Example", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                    }
                }
            }
        }

        /// <summary>
        /// Verwirft alle nderungen und stellt den letzten Eindeutigen Zustand wieder her.
        /// </summary>
        public void Reject()
        {
            // Wenn sich die Ansicht im Bearbeistungsmodus befindet ...
            if (_editable)
            {
                // nderungen abschlieen
                _binding.EndEdit();

                // Sperre freigeben 
                ApplicationServer.ReleaseLock("Repositories", SelectedRepositoryID.ToString());

                // nderungen verwerfen
                _stateContainer.Repositories.RejectChanges();

                // Schalter fr den Bearbeitungsmodus zurcksetzen
                _editable = false;

                // Symbolleistenbefehle aktualisieren
                RefreshToolbarButtons();

                // Gebundene Steuerelemente einfrben und entsperren
                ColorizeAndUnLockBoundControls();
            }
        }

        /// <summary>
        /// Speichert nderungen.
        /// </summary>
        public void Save()
        { 
            // Wenn sich die Ansicht im Bearbeistungsmodus befindet ...
            if (_editable)
            {
                // nderungen abschlieen
                _binding.EndEdit();

                try
                {
                    // Versuchen, nderungen an Lagern zu speichern
                    InventoryDataSet.RepositoriesDataTable savedRepositories = InventoryServiceProxy.SaveRepositories(_stateContainer.Repositories, null);

                    // Sperre freigeben 
                    ApplicationServer.ReleaseLock("Repositories", SelectedRepositoryID.ToString());

                    // Gespeicherte Datenstze neu lesen
                    _stateContainer.Repositories.Merge(savedRepositories);

                    // Schalter fr den Bearbeitungsmodus zurcksetzen
                    _editable = false;

                    // Symbolleistenbefehle aktualisieren
                    RefreshToolbarButtons();

                    // Gebundene Steuerelemente einfrben und entsperren
                    ColorizeAndUnLockBoundControls();
                }
                catch (Exception ex)
                { 
                    // Fehlermeldung anzeigen
                    MessageBox.Show(ex.Message, LanguageResource.Error, MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        /// <summary>
        /// Erzeugt ein neues Lager.
        /// </summary>
        public void CreateNewRepository()
        {
            // Wenn sich die Ansicht nicht im Bearbeitungsmodus befindet ...
            if (!_editable)
            {
                // Lagerdatensatz erzeugen
                InventoryDataSet.RepositoriesRow repository = _stateContainer.Repositories.NewRepositoriesRow();

                // Lagertschlssel vergeben
                repository.RepositoryID = Guid.NewGuid();

                // Versuchen, eine Sperre auf das Produkt zu bekommen 
                LockingInfo lockInfo = ApplicationServer.AcquireLock("Repositories", repository.RepositoryID.ToString());

                // Wenn die Sperre zugeteilt wurde ...
                if (lockInfo.Success)
                {
                    // Datensatz an Tabelle anfgen
                    _stateContainer.Repositories.AddRepositoriesRow(repository);

                    // Bearbeitungsmodus aktivieren
                    _editable = true;

                    // Zum letzten Datensatz springen
                    _binding.MoveLast();

                    // Gebundene Steuerelemente einfrben und (ent)sperren
                    ColorizeAndUnLockBoundControls();

                    // Symbolleistenbefehle aktualisieren
                    RefreshToolbarButtons();
                }
                else
                {
                    // Warnmeldung anzeigen
                    MessageBox.Show(lockInfo.Message, "Rainbirds n-Tier Example", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
                }
            }
        }

        #endregion

        #region Zugriff auf die Lagerverwaltung

        /// <summary>
        /// Ldt die aktuellen Lager-Datenstze in die Ansicht.
        /// </summary>
        private void LoadRepositories()
        {
            // Ggf. geladene Lager-Datenste verwerfen
            _stateContainer.Repositories.Rows.Clear();

            // Alle Lager-Datenstze abrufen
            InventoryDataSet.RepositoriesDataTable repositories = InventoryServiceProxy.GetAllRepositories();

            // Abgerufene Lager-Datenstze in den Status-Container schieben
            _stateContainer.Repositories.Merge(repositories);
        }        

        #endregion

        #region Formular-Ereignisprozeduren

        /// <summary>
        /// Wird ausgefhrt, wenn das Formular geladen wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void RepositoryEditor_Load(object sender, EventArgs e)
        {
            // Gebundene Steuerelemente sperren
            ColorizeAndUnLockBoundControls();

            // Symbolleistenbefehle aktualisieren
            RefreshToolbarButtons();

            // Lagerdaten laden und anzeigen
            LoadRepositories();
        }

        /// <summary>
        /// Wird ausgefhrt wenn Symbolleistenkopf "Neu" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _createRepositoryTool_Click(object sender, EventArgs e)
        {
            // Neues Lager erstellen
            CreateNewRepository();
        }

        /// <summary>
        /// Wird ausgefhrt wenn Symbolleistenkopf "Bearbeiten" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _editTool_Click(object sender, EventArgs e)
        {
            // Bearbeiten
            Edit();
        }

        /// <summary>
        /// Wird ausgefhrt wenn Symbolleistenkopf "Verwerfen" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _rejectChangesTool_Click(object sender, EventArgs e)
        {
            // Verwerfen
            Reject();
        }

        /// <summary>
        /// Wird ausgefhrt wenn Symbolleistenkopf "Speichern" geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _saveTool_Click(object sender, EventArgs e)
        {
            // Speichern
            Save();
        }

        /// <summary>
        /// Wird ausgefhrt wenn zu einem anderen Datensatz navigiert wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _binding_PositionChanged(object sender, EventArgs e)
        {
            // Symbolleistenbefehle aktualisieren
            RefreshToolbarButtons();
        }

        #endregion
    }
}