using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Rainbird.AppServer.API;
using Rainbird.Examples.NTier.Inventory.Contracts;

namespace Rainbird.Examples.NTier.Inventory.WindowsGUI
{
    /// <summary>
    /// Dialog zum Anlegen und Bearbeiten von Lagerbestnden.
    /// </summary>
    public partial class StockEditDialog : Form
    {
        #region Deklarationen

        // Stellvertreterobjekt fr den Zugriff auf den Lagerverwaltungsdienst.
        private IInventoryService _inventoryServiceProxy = null;

        // Schalter fr "Neuen Bestand"
        private bool _createNewStock = false;

        #endregion

        #region Konstruktoren

        /// <summary>
        /// Privater Standardkonstruktor.
        /// </summary>
        private StockEditDialog()
        {
            // Steuerelemente vorbereiten
            InitializeComponent();
        }

        #endregion

        /// <summary>
        /// ffnet einen Dialog, um einen neuen Bestand fr ein bestimmtes Produkt anzulegen.
        /// </summary>
        /// <param name="productID">Produktschlssel</param>
        public static void ShowCreateStockDialog(Guid productID)
        { 
            // Dialog erzeugen
            StockEditDialog dialog = new StockEditDialog();

            // Wenn Neuanlegen mglich ist ... 
            if (dialog.PrepareNewStock(productID))
                // Dialog anzeigen
                dialog.ShowDialog();
        }

        /// <summary>
        /// ffnet einen Dialog, um die Parameter eines bestehenden Bestandes zu aktualisieren.
        /// </summary>
        /// <param name="stockID">Bestandsschlssel</param>
        public static void ShowUpdateStockParametersDialog(Guid stockID)
        {
            // Dialog erzeugen
            StockEditDialog dialog = new StockEditDialog();

            // Wenn der Bestand geladen werden konnte ... 
            if (dialog.PrepareStockParameterUpdate(stockID))
                // Dialog anzeigen
                dialog.ShowDialog();
        }

        #region Formular Ereignisprozeduren

        /// <summary>
        /// Wird ausgefhrt, wenn der Dialog geladen wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void StockEditDialog_Load(object sender, EventArgs e)
        {

        }

        #endregion

        /// <summary>
        /// Gibt das Stellvertreterobjekt fr den Zugriff auf die Lagerverwaltung zurck
        /// und stellt vorher sicher, dass eine Verbindung hergestellt ist.
        /// </summary>
        private IInventoryService InventoryServiceProxy
        {
            get
            {
                // Wenn noch kein Stellvertreterobjekt erzeugt wurde ...
                if (_inventoryServiceProxy == null)
                    // Verbindung zum Lagerverwaltungsdienst herstellen
                    _inventoryServiceProxy = ServiceFactory<IInventoryService>.CreateProxy();

                // Stellvertreterobjekt zurckgeben
                return _inventoryServiceProxy;
            }
        }
        
        /// <summary>
        /// Bereitet den Dialog fr die Eingabe von Parametern fr einen neuen Lagerbestand vor.
        /// </summary>
        /// <param name="productID">Produktschlssel</param>
        /// <returns>Wahr, wenn ein Lagerbestand angelegt werden kann, ansonsten Falsch</returns>
        private bool PrepareNewStock(Guid productID)
        {
            // Lger abrufen, die noch keinen Bestand des angegebenen Produkts enthalten
            DataTable freeRepositories = InventoryServiceProxy.FindRepositoriesWithoutProductOnStock(productID);

            // Wenn mindestens ein Lager verfgbar ist ...
            if (freeRepositories.Rows.Count > 0)
            {
                // Dropdown-Liste des Lagerauswahl-Kombinationsfeldes fllen
                _repositoryIDCombo.DisplayMember = "Name";
                _repositoryIDCombo.ValueMember = "RepositoryID";
                _repositoryIDCombo.DataSource = freeRepositories;

                // Bearbeitbare Steuerelemente gelb einfrben
                _repositoryIDCombo.BackColor = SystemColors.Info;
                _binLocationBox.BackColor = SystemColors.Info;
                _estimatedStockBox.BackColor = SystemColors.Info;
                _reorderPointBox.BackColor = SystemColors.Info;
                
                // Nicht bearbeitbare Steuerelemente deaktivieren
                _actualStockBox.Enabled = false;

                // Ggf. geladene Bestnde entsorgen
                _stockData.Stocks.Rows.Clear();

                // Neuen Bestandsdatensatz erzeugen
                InventoryDataSet.StocksRow stock = _stockData.Stocks.NewStocksRow();

                // Lagerschlssel erzeugen
                stock.StockID = Guid.NewGuid();

                // Produktschlssel zuweisen
                stock.ProductID = productID;

                // Datensatz anfgen
                _stockData.Stocks.AddStocksRow(stock);

                // Zum neuen Datensatz navigieren
                _binding.MoveLast();

                // Fokus auf Lagerauswahl-Kombinationsfeldes setzen
                _repositoryIDCombo.Focus();

                // Dropdown-Liste ausklappen
                _repositoryIDCombo.DroppedDown = true;

                // Schalter fr "Neuen Bestand" setzen
                _createNewStock = true;

                // Wahr zurckgeben
                return true;
            }
            else
            { 
                // Warnung anzeigen
                MessageBox.Show("Kein Lager verfgbar! Entweder sind keine Lger vorhanden oder der Artikel ist bereits in allen Lgern eingebucht.", "n-Tier Example", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }            
            // Falsch zurckgeben
            return false;
        }

        /// <summary>
        /// Bereitet den Dialog fr die Eingabe von Parametern fr einen bestehenden Lagerbestand vor.
        /// </summary>
        /// <param name="stockID">Bestandsschlssel</param>
        /// <returns>Wahr, wenn der Lagerbestand gefunden wurde, ansonsten Falsch</returns>
        private bool PrepareStockParameterUpdate(Guid stockID)
        {
            // Bestand abrufen
            InventoryDataSet.StocksDataTable existingStock = InventoryServiceProxy.GetStock(stockID);

            // Wenn der Bestand gefunden wurde ...
            if (existingStock.Rows.Count==1)
            {
                // Lger abrufen
                InventoryDataSet.RepositoriesDataTable repositories = InventoryServiceProxy.GetAllRepositories();

                // Dropdown-Liste des Lagerauswahl-Kombinationsfeldes fllen
                _repositoryIDCombo.DisplayMember = "Name";
                _repositoryIDCombo.ValueMember = "RepositoryID";
                _repositoryIDCombo.DataSource = repositories;

                // Bearbeitbare Steuerelemente gelb einfrben
                _binLocationBox.BackColor = SystemColors.Info;
                _estimatedStockBox.BackColor = SystemColors.Info;
                _reorderPointBox.BackColor = SystemColors.Info;

                // Nicht bearbeitbare Steuerelemente deaktivieren
                _repositoryIDCombo.Enabled = false;
                _actualStockBox.Enabled = false;

                // Ggf. geladene Bestnde entsorgen
                _stockData.Stocks.Rows.Clear();

                // Bestand in DataSet bernehmen
                _stockData.Stocks.Merge(existingStock);

                // Zum neuen Datensatz navigieren
                _binding.MoveLast();

                // Schalter fr "Neuen Bestand" zurcksetzen
                _createNewStock = false;

                // Wahr zurckgeben
                return true;
            }
            else
            {
                // Warnung anzeigen
                MessageBox.Show("Der angegebene Lagerbestand wurde nicht gefunden.", "n-Tier Example", MessageBoxButtons.OK, MessageBoxIcon.Exclamation);
            }
            // Falsch zurckgeben
            return false;
        }

        /// <summary>
        /// Speichert nderungen am geladenen Lagerbestand bzw. legt einen neuen Lagerbestand an.
        /// </summary>
        /// <returns>Wahr, wenn erfolgreich gespeichert wurde, ansonsten Falsch</returns>
        private bool Save()
        {
            // Eingaben des Benutzers abrufen
            InventoryDataSet.StocksRow stock = _stockData.Stocks[0];

            try
            {
                // Wenn der Schalter fr "Neuen Bestand" gesetzt ist ...
                if (_createNewStock)
                    // Bestand anlegen
                    InventoryServiceProxy.CreateStock(stock.RepositoryID, stock.ProductID, stock.BinLocation, stock.EstimatedStock, stock.ReorderPoint, null);
                else
                    // Parameter des bestehendes Bestandes aktualisieren
                    InventoryServiceProxy.UpdateStockParameters(stock.StockID, stock.BinLocation, stock.EstimatedStock, stock.ReorderPoint, null);
            
                // Wahr zurckgeben
                return true;
            }
            catch (Exception ex)
            { 
                // Fehlermeldung ausgeben
                MessageBox.Show(ex.Message, "Rainbird n-Tier Example", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            // Falsch zurckgeben
            return false;
        }

        /// <summary>
        /// Wird ausgefhrt, wenn auf OK geklickt wird.
        /// </summary>
        /// <param name="sender">Herkunftsobjekt</param>
        /// <param name="e">Ereignisargumente</param>
        private void _acceptButton_Click(object sender, EventArgs e)
        {
            // Wenn erfolgreich gespeichert wurde ...
            if (Save())
                // Dialog schlieen
                Close();
        }
    }
}